Unit Maths;

Interface

uses Windows,SysUtils,math;

const
     MaxVecteur = 8000;
     RadianVersDegre = 57.29577951308232090;
     DegreVersRadian = 0.017453292519943296;

type
     IndiceVecteur = 0..MaxVecteur;
     Vecteur = array[IndiceVecteur] of double;
     VecteurInt = array[IndiceVecteur] of integer;
     EsincError = class(Exception);

const
     MaxReal : double = 1e+35;
     MinReal : double = 1e-35;
     ProcheReal : double = 1e-6;
     Loge = 0.434294481903251828;
     Ln10 = 2.302585092994045680;

Function Dix(x : integer) : double;
Function Log(x : double) : double;
Function MyInt(x : double) : double;
Function Plancher(x : double) : LongInt;
Function Plafond(x : double) : LongInt;
Function Fract(x : double) : double;
Function Puiss(X,Y : double): double; { X^Y }
Function Atan2(im,re : double) : double;
function Puiss2Inf(N : integer) : integer;
   { Puissance de 2 infrieure ou gale  N }
function Puiss2Sup(N : integer) : integer;
  { Puissance de 2 suprieure ou gale  N }

Procedure VerifMinMaxInt(var mini,maxi : integer);
Procedure VerifMinMaxReal(var mini,maxi : double);
procedure DeriveeGaussVect(const X,Y:Vecteur;N : integer;
   var Yder,Ysec : Vecteur);
function DeriveeGaussPoint(const X,Y:Vecteur;N,Nder : integer;
        Xval: double): double;
procedure AccelerationVect(const X,Y:Vecteur;N : integer;
   var acc : Vecteur);
function AccelerationPoint(const X,Y:vecteur;Nbre : integer) : double;
// procedure ExtrapoleSinc(const data : vecteur;var newData : vecteurSurEch;N : integer);

Implementation

Function PuissEnt(X:double;N:Integer): double;

  Function PuissDeux(N:integer):double; { N>=0 }
  var Y : double;
  begin
     if N=0
        then PuissDeux := 1
        else begin
           Y := PuissDeux(N div 2);
	   Y := Sqr(Y);
 	   if (N mod 2)=0
	      then PuissDeux := Y
	      else PuissDeux := X*Y;
       end;
  end;{ puissDeux }

begin {PuissEnt}
    If N<0
       then PuissEnt := 1/PuissDeux(-N)
       else PuissEnt := PuissDeux(N)
end;{PuissEnt}

function Dix(x : integer) : double;
Begin
    Dix := PuissEnt(10,X)
end;

Function Log(X:double):double;
Begin
   Log := Ln(X)*loge
End;

function Plancher(x : double) : LongInt;
var zut : LongInt;
begin
     Zut := trunc(x);
     if (x>=0) or (zut=x)
        then Plancher := zut
        else Plancher := zut-1
end;

function MyInt(x : double) : double;
var zut : double;
begin
    Zut := int(x);
    if (x>=0) or (zut=x)
       then Myint := zut
       else Myint := zut-1
end;

function Plafond(x : double) : LongInt;
var zut : LongInt;
begin
     Zut := trunc(x);
     if (x<0) or (Zut=x)
       then Plafond := zut
       else Plafond := zut+1
end;

function Fract(x : double) : double;
var zut : double;
begin
     Zut := frac(x);
     if (x>=0) or (zut=0)
        then Fract := zut
        else Fract := zut+1
end;

Function Puiss(X,Y : double): double; { X^Y }
var Z : integer;
Begin
    Z := round(Y);
    If Abs(Y-Z)<ProcheReal
         Then Puiss := PuissEnt(X,Z)
         Else Puiss := Exp(Ln(X)*Y)
end;{Puiss}

Procedure VerifMinMaxInt(var mini,maxi : integer);
var z : integer;
begin
   if Maxi<Mini then begin
	  z := Maxi;
	  Maxi := Mini;
	  Mini := z;
   end;
end;

Procedure VerifMinMaxReal(var mini,maxi : double);
var z : double;
begin
   if Maxi<Mini then begin
	  z := Maxi;
	  Maxi := Mini;
	  Mini := z;
   end;
end;

Function AngleUtilisateur(alpha : double) : double;
{ donne l'angle en  correspondant  l'angle exprim en radian }
begin
   result := alpha*RadianVersDegre
end;


Function Atan2(im,re:double) : double;
{ z = re + j*im }
var valeur : double;
begin
   try
   valeur := arcTan(im/re);
   if re<0 then if im<0
       then valeur := valeur-pi
       else valeur := valeur+pi;
   atan2 := AngleUtilisateur(valeur);
   except
         on EZeroDivide do begin
            if im<0
	       then valeur := -pi/2
               else valeur := pi/2;
            atan2 := AngleUtilisateur(valeur)
        end;
   end;
end;

function Puiss2Inf(N : integer) : integer;
begin
    result := 1;
    while result<N do result := 2*result;
    if result>N then result := result div 2;
end;

function Puiss2Sup(N : integer) : integer;
begin
    result := Puiss2Inf(N);
    if result<N then result := 2*result
end;

Function sinc(x : double) : double;
begin
   x := x*pi;
   sinc := sin(x)/x;
end;

{
procedure ExtrapoleSinc(const data : vecteur;var newData : vecteurSurEch;N : integer);
const a = 3;
var kernel : array[-scale*a..+scale*a] of double;
    i,j,k,j0 : integer;
    x : array[0..scale-1] of double;
    y : double;
begin
    for i := 1 to scale*a do begin
        y := i/scale;
        kernel[i] := sinc(y)*sinc(y/a);
        kernel[-i] := kernel[i];
    end;
    kernel[0] := 1;
    for i := 0 to a-2 do // linaire
        for j := 0 to pred(scale) do
            newData[j+i*scale] := data[i]+j*(data[i+1]-data[i])/scale;
    for i := pred(a) to (N-a-1) do begin
        for j := 0 to pred(scale) do
            x[j] := 0;
        for k := i-a+1 to i+a do begin
            j0 := scale*(k-i);
            for j := 0 to pred(scale) do
                x[j] := x[j]+kernel[j0-j]*data[k];
        end;
        for j := 0 to pred(scale) do
            newData[j+i*scale] := x[j];
    end;
    for i := (N-a) to (N-2) do // linaire
        for j := 0 to pred(scale) do
            newData[j+i*scale] := data[i]+j*(data[i+1]-data[i])/scale;
    newData[(N-1)*scale] := data[N-1];
//    newN := (N-1)*scale + 1;
end;
}

{$I gauss}

end.

