unit uniteKer;

interface

uses orphyLabUtil, maths, SysUtils, Math;

type

   TuniteDeBase = (_metre, _kilogramme, _seconde, _ampere, _kelvin, _mole, _candela);
   EUniteError  = class(Exception);
   TuniteSansDim = (tuVide, tuDegre, tuRadian, tuDecibel);

   Tunite = class
   private
      FnomUnite:        string;
      SansDim:          TuniteSansDim;
      Dimension:        array[TuniteDeBase] of shortInt;
      Puissance:        integer;
      NbreDecimale : integer;
      procedure SetNomUnite(const nu: string);
      procedure MulUnite(op: char; U0: Tunite; puiss: shortint);
      procedure PlusUnite(U0: Tunite);
      procedure InverseUnite;
   public
      nom: string;
      NbreDecimaleFixe : integer;
      property nomUnite: string Read FnomUnite Write SetNomUnite;
      constructor Create;
      procedure setUniteComplet(const nu: string;
         p: shortint; d1, d2, d3, d4, d5, d6, d7: shortint);
      procedure Init;
      procedure RecopieUnite(U: Tunite);
      procedure Assign(U: Tunite);
      function NomAff(puissValeur: shortint): string;
      function NomAxe(exposant: double): string;
      function formatUnite(nombre: double): string;
      function formatSimple(Nombre: double): string;
      function formatUniteBrute(Nombre: double): string;
      function formatUniteDouble(mini, maxi: double): string;
      function NomValeurUnite(nombre: double): string; virtual;
   end;

const
   NbreUniteConnue  = 31;
   NbreUniteToleree = 8;
   IndexRadian      = 2;
   IndexCelsius     = 3;
   NomBase: array[TuniteDeBase] of string =
      ('m', 'kg', 's', 'A', 'K', 'mol', 'cd');
   NomSansDim: array[TuniteSansDim] of string =
      ('', '', 'rad', 'dB');
   UneHeure = 3600;
   UnJour = UneHeure*24;

var
   UniteConnue:        array[1..NbreUniteConnue] of Tunite;
   UniteToleree:       array[1..NbreUniteToleree] of Tunite;
   UniteNulle, UTemps, UFrequence : Tunite;

function ChaineDuree(t: double): string;
function ChaineFrequence(t: double): string;
function ChaineHorloge(t: double): string;
function DateTimeToFloat(nombre: string): double;

implementation

procedure Tunite.InverseUnite;
var
   i: TuniteDeBase;
begin
   Puissance := -puissance;
   for i := _Metre to _Candela do
      dimension[i] := -dimension[i];
   sansDim := tuVide;
end; {InverseUnite}

procedure Tunite.PlusUnite(U0: Tunite);
var
   i: TuniteDeBase;
begin
   Puissance := puissance + U0.puissance;
   for i := _Metre to _Candela do
      dimension[i] := dimension[i] + U0.dimension[i];
   if U0.sansDim <> tuVide then
      if sansDim = tuVide then
         sansDim := U0.sansDim
      else
         sansDim := tuVide;
end; {PlusUnite}

procedure Tunite.MulUnite(op: char; U0: Tunite; puiss: shortint);
var
   i: TuniteDeBase;
begin
   if puiss <> 1 then
      sansDim := tuVide;
   case op of
      '*': begin
         Puissance := U0.Puissance * puiss;
         for i := _Metre to _Candela do
            dimension[i] := U0.dimension[i] * puiss;
      end;
      '/': begin
         if (U0.puissance mod puiss <> 0) then exit;
         Puissance := U0.Puissance div puiss;
         for i := _Metre to _Candela do begin
            if (U0.dimension[i] mod puiss <> 0) then exit;
            dimension[i] := U0.dimension[i] div puiss;
         end;
      end;
   end; { case }
end;{MulUnite}

procedure Tunite.SetNomUnite(const nu: string);
{ Pour les units dont le nomUnite est impos par l'utilisateur }
var
   Ucourant: Tunite;
   Inverse:  boolean;
   LongUnite: integer;

   procedure AnalysePrefixe(var index: integer);
   begin
      if FnomUnite[index] = '/' then begin
         Inc(index);
         inverse := True;
      end;
      if (index < LongUnite) and charinset(FnomUnite[index],caracPrefixe) and
         charinset(FnomUnite[succ(index)],majuscule + minuscule + ['']) then
      begin
         case FnomUnite[index] of
            'a': Ucourant.puissance  := -18;
            'f': Ucourant.puissance  := -15;
            'p': Ucourant.puissance  := -12;
            'n': Ucourant.puissance  := -9;
            '': Ucourant.puissance := -6;
            'm': Ucourant.puissance  := -3;
            'c': Ucourant.puissance  := -2;
            'd': Ucourant.puissance  := -1;
            'h': Ucourant.puissance  := +2;
            'k': Ucourant.puissance  := +3;
            'M': Ucourant.puissance  := +6;
            'G': Ucourant.puissance  := +9;
            'T': Ucourant.puissance  := +12;
            'P': Ucourant.puissance  := +15;
            'E': Ucourant.puissance  := +18;
         end;
         Inc(index);
      end;
   end;{AnalysePrefixe}

   procedure AnalyseCode(var index: integer);
   var
      trouve: boolean;
      j: integer;
      nomCourant: string;
      indexInit: integer;

      procedure OubliSeparateur;
      var
         j:  integer;
         UU: Tunite;
      begin
         index := indexInit;
         nomCourant := '';
         repeat
            nomCourant := nomCourant + FnomUnite[index];
            Inc(index);
            for j := 1 to NbreUniteConnue do begin
               UU := UniteConnue[j];
               trouve := nomCourant = UU.nomUnite;
               if trouve then begin
                  Ucourant.PlusUnite(UU);
                  exit;
               end;
            end;
            for j := 1 to NbreUniteToleree do begin
               UU := UniteToleree[j];
               trouve := nomCourant = UU.nomUnite;
               if trouve then begin
                  Ucourant.PlusUnite(UU);
                  exit;
               end;
            end;
         until (index > LongUnite) or not charinset(FnomUnite[index],Majuscule + Minuscule);
      end; { OubliSeparateur }

   var
      UU: Tunite;
   begin { AnalyseCode }
      nomCourant := '';
      indexInit  := index;
      repeat
         nomCourant := nomCourant + FnomUnite[index];
         Inc(index);
      until (index > longUnite) or not charinset(FnomUnite[index],Majuscule + Minuscule);
      for j := 1 to NbreUniteConnue do begin
         UU := UniteConnue[j];
         trouve := nomCourant = UU.nomUnite;
         if trouve then begin
            Ucourant.PlusUnite(UU);
            exit;
         end;
      end;
      for j := 1 to NbreUniteToleree do begin
         UU := UniteToleree[j];
         trouve := nomCourant = UU.nomUnite;
         if trouve then begin
            Ucourant.PlusUnite(UU);
            exit;
         end;
      end;
      oubliSeparateur;
   end; { AnalyseCode }

   procedure AnalyseSuffixe(var index: integer);
   var
      negatif, kilogramme: boolean;
      z: shortint;
      i: TuniteDeBase;
   begin
      if index > longUnite then begin
         if inverse then
            Ucourant.inverseUnite;
         exit;
      end;
      if FnomUnite[index] = '^' then begin
         Inc(index);
         if (index > LongUnite) then exit;
      end;
      negatif := False;
      if charinset(FnomUnite[index],['-', '+']) then begin
         negatif := charinset(FnomUnite[index],['-']);
         Inc(index);
         if (index > LongUnite) then exit;
      end;
      kilogramme := True;
      for i := _Metre to _Candela do
         if i = _kilogramme then
            kilogramme := kilogramme and (Ucourant.dimension[_kilogramme] <> 0)
         else
            kilogramme := kilogramme and (Ucourant.dimension[i] = 0);
      { cas  faire }
      if charinset(FnomUnite[index],chiffre + chiffrePuiss) then begin
         z := caracToChiffre(FnomUnite[index]);
         if negatif then
            z := -z;
         Inc(index);
      end
      else
         z := 1;
      if inverse then
         z := -z;
      Ucourant.MulUnite('*', Ucourant, z);
   end; { AnalyseSuffixe }

var
   i: TuniteDeBase;
   index, j: integer;
   stValeur: string;
begin { SetNomUnite }
   try { Erreur Unite }
      FnomUnite := nu;
      puissance := 0;
      if (nu='hPa') or (nu='tr/min') or (nu='mS/cm') then exit;
      for i := _Metre to _Candela do
         dimension[i] := 0;
      longUnite := Length(FnomUnite);
      for j := 1 to NbreUniteConnue do begin
         Ucourant := UniteConnue[j];
         if FnomUnite = Ucourant.nomUnite then begin
            RecopieUnite(Ucourant);
            exit;
         end;
      end;
      for j := 1 to NbreUniteToleree do begin
         Ucourant := UniteToleree[j];
         if FnomUnite = Ucourant.nomUnite then begin
            RecopieUnite(Ucourant);
            exit;
         end;
      end;
      index := 1;
      inverse := False;
      if Pos('1E', FNomUnite) = 1 then begin
         index := 3;
         while (index < LongUnite) and charinset(FnomUnite[index],Chiffre + ['-', '+']) do
            Inc(index);
         if index > 3 then begin
            stValeur  := copy(FnomUnite, 1, pred(index));
            puissance := StrToInt(StValeur);
         end;
         if FnomUnite[index] = ' ' then
            Inc(index);
      end; { 1E+xx }
      Ucourant := Tunite.Create;
      while (index <= LongUnite) do begin
         Ucourant.init;
         AnalysePrefixe(index);
         AnalyseCode(index);
         AnalyseSuffixe(index);
         PlusUnite(Ucourant);
         if (index < LongUnite) and (FnomUnite[index] = '.') then
            Inc(index); { sparateur }
      end;
      Ucourant.Free;
   except
      on EUniteError do
   end;
end; { SetNomUnite }

function AjoutePuissDix(nom: string; puiss: shortint): string;
var
   dest: string;
begin
   if puiss = 0 then
      ajoutePuissDix := nom
   else begin
      dest := '10' + puissToStr(puiss) + ' ';
      if length(dest) + Length(nom) > 16 then
         ajoutePuissDix := dest + 'S.I.'
      else
         ajoutePuissDix := dest + nom;
   end;
end;

function Tunite.NomAff(puissValeur: shortint): string;
   { unit dont on connait le dveloppement en puissance }

   function PrefixeConnu(p: shortint): boolean;
   begin
      PrefixeConnu := (p >= -18) and (p <= 18) and ((p mod 3 = 0) or
         (p = -2) or (p = -1) or (p = 2));
   end;

   function AjoutePrefixe(Nom: string; P: shortint): string;
   const
      lettreUnite=
         ['A', 'C', 'F', 'H', 'J', 'K', 'L', 'N', 'P', 'S', 'T', 'V', 'W',
         'b', 'c', 'd', 'g', 'l', 'm', 'r', 's', OmegaMaj];
   var
      prefixe: char;
   begin
      if nom = '' then begin
         ajoutePrefixe := AjoutePuissDix('', P);
         exit;
      end;
      if (length(nom) > 1) and
         charInset(nom[1],caracPrefixe) and
         charInset(nom[2],lettreUnite) then begin
         case FnomUnite[1] of
            'a': P  := P - 18;
            'f': P  := P - 15;
            'p': P  := P - 12;
            'n': P  := P - 9;
            '': P := P - 6;
            'm': P  := P - 3;
            'c': P  := P - 2;
            'd': P  := P - 1;
            'h': P  := P + 2;
            'k': P  := P + 3;
            'M': P  := P + 6;
            'G': P  := P + 9;
            'T': P  := P + 12;
            'P': P  := P + 15;
            'E': P  := P + 18;
         end;
         nom := copy(nom, 2, length(nom));
      end;
      case P of
         -18: prefixe := 'a';
         -15: prefixe := 'f';
         -12: prefixe := 'p';
         -9: prefixe  := 'n';
         -6: prefixe  := '';
         -3: prefixe  := 'm';
         -2: prefixe  := 'c';
         -1: prefixe  := 'd';
         0: begin
            ajoutePrefixe := nom;
            exit;
         end;
         +2: prefixe  := 'h';
         +3: prefixe  := 'k';
         +6: prefixe  := 'M';
         +9: prefixe  := 'G';
         +12: prefixe := 'T';
         +15: prefixe := 'P';
         +18: prefixe := 'E';
         else begin
            ajoutePrefixe := AjoutePuissDix(nom, P);
            exit;
         end;
      end;
      ajoutePrefixe := prefixe + nom;
   end;

var
   i:  TuniteDeBase;
   OKnulle, OKconnue: boolean;
   j:  integer;
   UU: Tunite;
   NomPositif: string;
   CasDuMetre: boolean;
begin
   if (puissValeur = 0) then begin
      NomAff := FnomUnite;
      exit;
   end;
   OKnulle := True;
   for i := _Metre to _Candela do
      OKnulle := OKnulle and (dimension[i] = 0);
   if OKnulle then begin
      NomAff := AjoutePrefixe(FnomUnite, PuissValeur);
      exit;
   end;
   puissValeur := puissance + puissValeur;
   CasDuMetre  := (dimension[_metre] <> 0) and (dimension[_metre] <> 1);
   for i := _KiloGramme to _Candela do
      CasDuMetre := CasDuMetre and (dimension[i] = 0);
   if CasDuMetre and  { 10^-6 m^2 = (mm)^2}
      (puissValeur mod dimension[_metre] = 0) and
      prefixeConnu(puissValeur div dimension[_metre]) then begin
      NomPositif := AjoutePrefixe('m', puissValeur div dimension[_metre]);
      { Pb des units donnes en mL ? }
      NomPositif := NomPositif + PuissToStr(dimension[_metre]);
      NomAff := NomPositif;
      exit;
   end;
   for j := 1 to NbreUniteToleree do
      if FnomUnite = UniteToleree[j].nomUnite then begin
         Result := AjoutePrefixe(FnomUnite, puissValeur - puissance);
         exit;
      end;
   for j := 1 to NbreUniteConnue do begin
      UU := UniteConnue[j];
      for i := _Metre to _Candela do begin
         OKconnue := dimension[i] = UU.dimension[i];
         if not OKconnue then
            break;
      end;
      if OKconnue and (sansDim = tuVide) then begin
         Result := AjoutePrefixe(UU.nomUnite, puissValeur - UU.puissance);
         exit;
      end;
   end;
   for j := 1 to NbreUniteConnue do begin
      UU := UniteConnue[j];
      for i := _Metre to _Candela do begin
         if i = _Seconde then
            OKconnue := dimension[_seconde] = UU.dimension[_seconde] - 1
         else
            OKconnue := dimension[i] = UU.dimension[i];
         if not OKconnue then
            break;
      end;
      if OKconnue and
         (sansDim = tuVide) and
         (UU.nomUnite <> 'Hz') then begin
         { = ou <> ? }
         Result := AjoutePrefixe(UU.nomUnite, puissValeur - UU.puissance);
         Result := Result + 's' + PuissToStr(-1);
         exit;
      end;
   end;
   NomAff := AjoutePuissDix(FnomUnite, puissValeur);
end; { NomAff }

function Tunite.NomAxe(exposant: double): string;
begin
   Result := nomAff(round(log(exposant)));
   if Result <> '' then
      if pos('/', Result) > 0 then
         Result := nom + '(' + Result + ')'
      else
         Result := nom + '/' + Result
   else
      nomAxe := nom;
end;

constructor Tunite.Create;
begin
   Init;
end;

procedure InitUnite;
begin
   UniteConnue[1] := Tunite.Create;
   UniteConnue[1].setUniteComplet('m', 0, 1, 0, 0, 0, 0, 0, 0);
   UniteConnue[2] := Tunite.Create;
   UniteConnue[2].setUniteComplet('A', 0, 0, 0, 0, 1, 0, 0, 0);
   UniteConnue[3] := Tunite.Create;
   UniteConnue[3].setUniteComplet('g', -3, 0, 1, 0, 0, 0, 0, 0);
   UniteConnue[4] := Tunite.Create;
   UniteConnue[4].setUniteComplet('K', 0, 0, 0, 0, 0, 1, 0, 0);
   UniteConnue[5] := Tunite.Create;
   UniteConnue[5].setUniteComplet('mol', 0, 0, 0, 0, 0, 0, 1, 0);
   UniteConnue[6] := Tunite.Create;
   UniteConnue[6].setUniteComplet('cd', 0, 0, 0, 0, 0, 0, 0, 1);
   UniteConnue[7] := Tunite.Create;
   UniteConnue[7].setUniteComplet('H', 0, 2, 1, -2, -2, 0, 0, 0);
   UniteConnue[8] := Tunite.Create;
   UniteConnue[8].setUniteComplet('mol/l', +3, -3, 0, 0, 0, 0, 1, 0);
   UniteConnue[9] := Tunite.Create;
   UniteConnue[9].setUniteComplet('F', 0, -2, -1, 4, 2, 0, 0, 0);
   UniteConnue[10] := Tunite.Create;
   UniteConnue[10].setUniteComplet('N', 0, 1, 1, -2, 0, 0, 0, 0);
   UniteConnue[11] := Tunite.Create;
   UniteConnue[11].setUniteComplet(omegaMaj, 0, 2, 1, -3, -2, 0, 0, 0);
   UniteConnue[12] := Tunite.Create;
   UniteConnue[12].setUniteComplet('S', 0, -2, -1, 3, 2, 0, 0, 0);
   UniteConnue[13] := Tunite.Create;
   UniteConnue[13].setUniteComplet('T', 0, 0, 1, -2, -1, 0, 0, 0);
   UniteConnue[14] := Tunite.Create;
   UniteConnue[14].setUniteComplet('V', 0, 2, 1, -3, -1, 0, 0, 0);
   UniteConnue[15] := Tunite.Create;
   UniteConnue[15].setUniteComplet('Hz', 0, 0, 0, -1, 0, 0, 0, 0);
   UFrequence := UniteConnue[15];
   UniteConnue[16] := Tunite.Create;
   UniteConnue[16].setUniteComplet('s', 0, 0, 0, 1, 0, 0, 0, 0);
   UTemps := UniteConnue[16];
   UniteConnue[17] := Tunite.Create;
   UniteConnue[17].setUniteComplet('Pa', 0, -1, 1, -2, 0, 0, 0, 0);
   UniteConnue[18] := Tunite.Create;
   UniteConnue[18].setUniteComplet('J', 0, 2, 1, -2, 0, 0, 0, 0);
   UniteConnue[19] := Tunite.Create;
   UniteConnue[19].setUniteComplet('W', 0, 2, 1, -3, 0, 0, 0, 0);
   UniteConnue[20] := Tunite.Create;
   UniteConnue[20].setUniteComplet('C', 0, 0, 0, 1, 1, 0, 0, 0);
   UniteConnue[21] := Tunite.Create;
   UniteConnue[21].setUniteComplet('Wb', 0, 2, 1, -2, -1, 0, 0, 0);
   UniteConnue[22] := Tunite.Create;
   UniteConnue[22].setUniteComplet('lm', 0, 0, -2, 0, 0, 0, 0, 1);
   UniteConnue[23] := Tunite.Create;
   UniteConnue[23].setUniteComplet('N/m', 0, 0, 1, -2, 0, 0, 0, 0);
   UniteConnue[24] := Tunite.Create;
   UniteConnue[24].setUniteComplet('J/K', 0, 2, 1, -2, 0, -1, 0, 0);
   UniteConnue[25] := Tunite.Create;
   UniteConnue[25].setUniteComplet('J/(kg.K)', 0, 2, 0, -2, 0, -1, 0, 0);
   UniteConnue[26] := Tunite.Create;
   UniteConnue[26].setUniteComplet('V/m', 0, 1, 1, -3, -1, 0, 0, 0);
   UniteConnue[27] := Tunite.Create;
   UniteConnue[27].setUniteComplet('S/m', 0, -3, -1, 3, 2, 0, 0, 0);
   UniteConnue[28] := Tunite.Create;
   UniteConnue[28].setUniteComplet('W/(m.K))', 0, 1, 1, -3, 0, -1, 0, 0);
   UniteConnue[29] := Tunite.Create;
   UniteConnue[29].setUniteComplet('C.m', 0, 1, 0, 0, 0, 0, 0, 0);
   UniteConnue[30] := Tunite.Create;
   UniteConnue[30].setUniteComplet(OmegaMaj + 'm', 0, 3, 1, -3, -2, 0, 0, 0);
   UniteConnue[31] := Tunite.Create;
   UniteConnue[31].setUniteComplet('N.m', 0, 1, 0, 1, 1, 0, 0, 0);

   UniteToleree[1] := Tunite.Create; { litre }
   UniteToleree[1].setUniteComplet('L', -3, 3, 0, 0, 0, 0, 0, 0);
   UniteToleree[2] := Tunite.Create;
   UniteToleree[2].setUniteComplet('rad', 0, 0, 0, 0, 0, 0, 0, 0);
   UniteToleree[3] := Tunite.Create;
   UniteToleree[2].sansDim := tuRadian;
   UniteToleree[3].setUniteComplet('C', 0, 0, 0, 0, 0, 1, 0, 0);
   UniteToleree[4] := Tunite.Create;
   UniteToleree[4].setUniteComplet('bar', 5, -1, 1, -2, 0, 0, 0, 0);
   UniteToleree[5] := Tunite.Create;
   UniteToleree[5].setUniteComplet('dB', 0, 0, 0, 0, 0, 0, 0, 0);
   UniteToleree[5].sansDim := tuDecibel;
   UniteToleree[6] := Tunite.Create;
   UniteToleree[6].setUniteComplet('', 0, 0, 0, 0, 0, 0, 0, 0);
   UniteToleree[6].sansDim := tuDegre;
   UniteToleree[7] := Tunite.Create; { % }
   UniteToleree[7].setUniteComplet('%', 0, 0, 0, 0, 0, 0, 0, 0);
   UniteToleree[8] := Tunite.Create; { tour par minute }
   UniteToleree[8].setUniteComplet('tr/min', 0, 0, -1, 0, 0, 0, 0, 0);

   UniteNulle := Tunite.Create;
   UniteNulle.setUniteComplet('', 0, 0, 0, 0, 0, 0, 0, 0);
end;

procedure Tunite.SetUniteComplet(const nu: string; p: shortint;
   d1, d2, d3, d4, d5, d6, d7: shortint);
begin
   FnomUnite := nu;
   puissance := p;
   Dimension[_metre] := d1;
   Dimension[_kilogramme] := d2;
   Dimension[_seconde] := d3;
   Dimension[_ampere] := d4;
   Dimension[_kelvin] := d5;
   Dimension[_mole] := d6;
   Dimension[_Candela] := d7;
end;

procedure Tunite.Init;
var
   i: TuniteDeBase;
begin
   FnomUnite := '';
   Puissance := 0;
   sansDim := tuVide;
   for i := _Metre to _Candela do
      Dimension[i] := 0;
   nom := '';
   nbreDecimale := 2;
   nbreDecimaleFixe := -1;
end;

procedure Tunite.RecopieUnite(U: Tunite);
var
   i: TuniteDeBase;
begin
   FnomUnite := U.nomUnite;
   Puissance := U.puissance;
   sansDim := U.sansDim;
   for i := _Metre to _Candela do
      Dimension[i] := U.dimension[i];
end;

procedure Tunite.Assign(U: Tunite);
begin
   RecopieUnite(U);
   Nom := U.Nom;
end;

function Tunite.FormatUnite(Nombre: double): string;
var
   negatif: boolean;
   tampon: string;
   unite: coefUnite;
begin
   if isNan(nombre) then
      Result := ''
   else if abs(nombre) < 1e-13 then
      Result := '0' + nomAff(0)
   else begin
      negatif := nombre < 0;
      nombre := abs(nombre);
      unite := nulle;
      while (nombre >= 999) and (unite < infini) do begin
         nombre := nombre / 1000;
         Inc(unite);
      end; { nombre<1000 ou infini }
      while (nombre < 0.999) and (unite > zero) do begin
         nombre := nombre * 1000;
         Dec(unite);
      end; { nombre>=1 ou zero }
      if unite = infini then
         Result := '???'
      else if unite = zero then
         Result := '0' + nomAff(0)
      else begin
         tampon := FormatPortable(nombre);
         if negatif then
            tampon := '-' + tampon;
         Result := tampon + ' ' + nomAff(puissanceUnite[unite]);
      end;
   end;
end;

function Tunite.FormatUniteBrute(Nombre: double): string;
begin
   if isNan(nombre)
      then Result := ''
      else Result := FormatSimple(nombre) + nomAff(0)
end;

function Tunite.FormatSimple(Nombre: double): string;
begin
   if isNan(nombre)
      then Result := ''
      else if NbreDecimaleFixe=0
          then Result := IntToStr(round(nombre))
          else if NbreDecimaleFixe>0
             then Result := FloatToStrF(nombre, ffFixed, PrecisionPortable, nbreDecimaleFixe)
             else Result := FloatToStrF(nombre, ffFixed, PrecisionPortable, nbreDecimale)
end;

function Tunite.NomValeurUnite(Nombre: double): string;
begin
   NomValeurUnite := nom + '=' + formatUnite(nombre);
end;

function ChaineHorloge(t: double): string;
var
   strj, strh, strm, strs: string;
   d,j: longint;
begin
   d := Round(t);
   j := d div UnJour;
   if j>1 then begin
      strj := intTostr(j);
      strh :=inttostr((d div UneHeure) mod 24);
      Result := strj + 'j ' + strh + 'h';
      exit;
   end;
// moins d'un jour
   strh := inttostr(d div UneHeure);
   if strh[1] = ' ' then
      strh[1] := '0';
   strm := inttostr((D div 60) mod 60);
   if strm[1] = ' ' then
      strm[1] := '0';
   strs := inttostr(D mod 60);
   if strs[1] = ' ' then
      strs[1] := '0';
   Result := strh + ':' + strm + ':' + strs;
end;

Function ChaineFrequence(t : double) : string;
var f,f10 : double;
    exposant : integer;
begin
   f := 1/t;
   exposant := trunc(log(f));
   f10 := power(10,exposant-2);
   f := f10*round(f/f10);
   result := Ufrequence.formatUnite(f)
end;

Function ChaineDuree(t : double) : string;

function FormatLoc(Nombre: double): string;
var
   tampon: string;
   unite: coefUnite;
begin
   if isNan(nombre) then
      Result := ''
   else begin
      unite := nulle;
      while (nombre >= 999) and (unite < infini) do begin
         nombre := nombre / 1000;
         Inc(unite);
      end; { nombre<1000 ou infini }
      while (nombre < 0.999) and (unite > zero) do begin
         nombre := nombre * 1000;
         Dec(unite);
      end; { nombre>=1 ou zero }
      tampon := FormatTimeCourt(nombre);
      Result := tampon + ' '+Utemps.nomAff(puissanceUnite[unite]);
      end;
end;

var Njour : integer;
begin
   if t<5*60 { 5 minutes }
      then result := formatLoc(t)
      else begin
         if t>2*UneHeure then t := round(t/15)*15; { > 2 h  ; arrondi 15 s }
         if t>12*UneHeure then t := round(t/60)*60; { > 12 h ; arrondi 1 min}
         Njour := trunc(t/UnJour);
         if Njour>0
            then result := intToStr(Njour)+'j '+FormatDateTime('hh"h"',frac(t/UnJour))
            else if t>=UneHeure
                then result := FormatDateTime('hh"h"mm',t/UnJour)
                else result := FormatDateTime('h:mm:ss',t/UnJour)
                // h obligatoire sinon n'importe quoi, merci Bill
      end;
end;

procedure LibereUnite;
var
   i: integer;
begin
   for i := 1 to NbreUniteConnue do
      UniteConnue[i].Free;
   for i := 1 to NbreUniteToleree do
      UniteToleree[i].Free;
   UniteNulle.Free;
end;

function Tunite.FormatUniteDouble(mini, maxi: double): string;
var
   unite: coefUnite;
   chifAv, chifAp: integer;

   procedure ChercheFormat(nombre: double);
   begin
      nombre := abs(nombre);
      unite  := nulle;
      while (nombre >= 999) and (unite < infini) do begin
         nombre := nombre / 1000;
         Inc(unite);
      end; { nombre<1000 ou infini }
      while (nombre < 0.999) and (unite > zero) do begin
         nombre := nombre * 1000;
         Dec(unite);
      end; // nombre>=1 ou zero
      if Nombre < 10 then
         chifAp := 1
      else
         chifAp := 0;
      chifAv := 5 - chifAp;
   end;

begin
   if nomUnite = 'hPa' then
      Result := IntToStr(round(mini)) + '..' + IntToStr(round(maxi)) + ' hPa'
   else if nomUnite = 'tr/min' then
      Result := IntToStr(round(maxi)) + ' tr/min'
   else if nomUnite = 'mS/cm' then
      Result := IntToStr(round(maxi)) + ' mS/cm'
   else begin
      if abs(maxi) > abs(mini)
         then ChercheFormat(maxi)
         else ChercheFormat(mini);
      Result := FloatToStrF(mini, ffFixed, precisionPortable, chifAp) + ' +' +
         FloatToStrF(maxi, ffFixed, precisionPortable, chifAp) + ' ' +
         nomAff(puissanceUnite[unite]);
   end;
   if maxi<10
      then nbreDecimale := 3
      else if maxi<100
           then nbreDecimale := 2
           else nbreDecimale := 1;
end;

function PosN(const C: string; S: string; const K: integer): integer;
var
   P: integer;
begin
   Result := 0;
   if K > 0 then
      Delete(S, 1, K);
   P := Pos(C, S);
   if P > 0 then
      Result := K + P;
end;

function DateTimeToFloat(nombre: string): double;
var
   i, Njour: integer;
begin
   Result := 0;
   trimComplet(nombre);
   i := pos(':', nombre);
   if i > 0 then begin
      i := pos('j', nombre);
      Njour := 0;
      if i > 0 then begin
         if i > 1 then begin
            try
               Njour := StrToInt(copy(nombre, 1, pred(i)));
            except
            end;
         end; { i>1 }
         Delete(nombre, 1, i);
      end { i>0 }
      else begin
         i := posN(':', nombre, 2);
         if i = 0 then
            nombre := '0:' + nombre;
      end;
      try
         Result := (StrToTime(nombre) + Njour) * 60 * 60 * 24;
      except
      end;
   end {i>0}
   else begin
      i := pos('j', nombre);
      if i > 0 then begin
         if i > 1 then begin
            try
               Result := StrToInt(copy(nombre, 1, pred(i))) * 60 * 24 * 60;
            except
            end;
         end;
         Delete(nombre, 1, i);
      end;
      i := pos('h', nombre);
      if i > 0 then begin
         if i > 1 then begin
            try
               Result := Result + StrToInt(copy(nombre, 1, pred(i))) * 60 * 60;
            except
            end;
         end;
         Delete(nombre, 1, i);
      end;
      i := pos('min', nombre);
      if i > 0 then begin
         if i > 1 then begin
            try
               Result := Result + StrToFloatWin(copy(nombre, 1, pred(i))) * 60;
            except
            end;
         end;
         Delete(nombre, 1, i + 2);
      end;
      i := pos('m', nombre);
      if i > 0 then begin
         if i > 1 then begin
            try
               Result := Result + StrToFloatWin(copy(nombre, 1, pred(i))) * 1e-3;
            except
            end;
         end;
         Delete(nombre, 1, i);
      end;
      i := pos('', nombre);
      if i > 0 then begin
         if i > 1 then begin
            try
               Result := Result + StrToFloatWin(copy(nombre, 1, pred(i))) * 1e-6;
            except
            end;
         end;
         Delete(nombre, 1, i);
      end;
      i := pos('s', nombre);
      if i > 0 then
         Delete(nombre, i, 1);
      if nombre <> '' then begin
         try
            Result := Result + strToFloatWin(nombre);
         except
         end;
      end;
      if Result = 0 then
         Result := 0.1;
   end;
end;

initialization
   InitUnite

finalization
   LibereUnite
end.

