unit voieCalcCfg;

interface

uses Windows, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, sysUtils, Dialogs, clipbrd, keysu, math,
  system.UITypes,
  orphyLabUtil, orphyLabDef, orphyLabGraphe, orphyLabMainU, maths,
  orphyLabConst, ImgList, ComCtrls, gbf_syntaxe, aidekey;

type
  TvoieCalcDlg = class(TForm)
    OKBtn: TBitBtn;
    GrandeurGB: TGroupBox;
    EditNom: TEdit;
    EditUnite: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Bevel1: TBevel;
    DesactiveBtn: TBitBtn;
    EtalonnageGB: TGroupBox;
    HelpBtn: TBitBtn;
    ImagePoint: TImageList;
    EditSignif: TEdit;
    Label3: TLabel;
    AffichageGB: TGroupBox;
    CouleurCombo: TColorBox;
    LabelAide: TLabel;
    ExpressionEdit: TLabeledEdit;
    EditMini: TLabeledEdit;
    EditMaxi: TLabeledEdit;
    PointCombo: TComboBoxEx;
    Memo1: TMemo;
    AideBtn: TSpeedButton;
    ListGrandeurs: TListBox;
    Label4: TLabel;
    NbreDecimaleLabel: TLabel;
    NbreDecimaleSE: TSpinEdit;
    FormatFixeCB: TCheckBox;
    procedure FormActivate(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure DesactiveBtnClick(Sender: TObject);
    procedure EditNomChange(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure EditNomKeyPress(Sender: TObject; var Key: Char);
    procedure EditMiniKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditMaxiKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditMiniExit(Sender: TObject);
    procedure EditMaxiKeyPress(Sender: TObject; var Key: Char);
    procedure EditMiniKeyPress(Sender: TObject; var Key: Char);
    procedure AideBtnClick(Sender: TObject);
    procedure ListGrandeursClick(Sender: TObject);
    procedure EditMiniChange(Sender: TObject);
    procedure EditMaxiChange(Sender: TObject);
    procedure CouleurComboChange(Sender: TObject);
    procedure PointComboChange(Sender: TObject);
    procedure ExpressionEditChange(Sender: TObject);
    procedure FormatFixeCBClick(Sender: TObject);
  private
    oldActive : boolean;
  public
    modif : boolean;
  end;

var
  voieCalcDlg: TvoieCalcDlg;

implementation

{$R *.DFM}

procedure TvoieCalcDlg.FormActivate(Sender: TObject);
var voie : indiceVoie;
begin with voieCalculee do begin
          EditNom.text := Nom;
          EditUnite.text := NomUnite;
          EditSignif.text := Signif;
          LabelAide.caption := Aide;
          oldActive := active;
          caption := 'Grandeur calcule';
          CouleurCombo.selected := couleur;
          PointCombo.itemIndex := ord(motif);
          EditMini.text := FormatPortable(mini);
          EditMaxi.text := FormatPortable(maxi);
          ExpressionEdit.text := expression;
          modif := false;
          formatFixeCB.Checked := NbreDecimaleFixe>=0;
          NbreDecimaleSE.value := NbreDecimaleFixe;
          NbreDecimaleSE.Visible := FormatFixeCB.checked;
          NbreDecimaleLabel.visible := FormatFixeCB.checked;
          ListGrandeurs.Items.Clear;
          for voie := voie0 to graphe.voieAnaMax do
              with graphe.voieOrphy[voie] do
            if active then
               ListGrandeurs.items.add(nom);
end end;

procedure TvoieCalcDlg.FormatFixeCBClick(Sender: TObject);
begin
  NbreDecimaleSE.Visible := FormatFixeCB.checked;
  NbreDecimaleLabel.visible := FormatFixeCB.checked;
end;

procedure TvoieCalcDlg.OKBtnClick(Sender: TObject);
begin
     modif := modif or not oldActive;
     with voieCalculee do begin
          active := expressionEdit.text<>'';
          Nom := EditNom.text;
          NomUnite := EditUnite.text;
          Signif := EditSignif.text;
          maxi := strToFloatWin(EditMaxi.text);
          if isNan(maxi) then maxi := 10;
          mini := StrToFloatWin(EditMini.text);
          if isNan(mini) then mini := -maxi;
          setMinMax;
          if FormatFixeCB.checked
              then NbreDecimaleFixe := NbreDecimaleSE.value
              else NbreDecimaleFixe := -1;
          couleur := CouleurCombo.selected;
          motif := Tmotif(PointCombo.itemIndex);
          if active then begin
             expression := expressionEdit.text;
             if not active then begin
                 MessageDlg('Erreur de syntaxe', mtError, [mbOK, mbHelp], 19);
                 modalResult := mrNone;
             end;
          end;
     end;
end;

procedure TvoieCalcDlg.PointComboChange(Sender: TObject);
begin
     modif := true;
end;

procedure TvoieCalcDlg.AideBtnClick(Sender: TObject);
begin
  syntaxeDlg := TsyntaxeDlg.create(self);
  syntaxeDlg.showModal;
  syntaxeDlg.free;
end;

procedure TvoieCalcDlg.CouleurComboChange(Sender: TObject);
begin
     modif := true;
end;

procedure TvoieCalcDlg.DesactiveBtnClick(Sender: TObject);
begin
      modif := oldActive;
      voieCalculee.active := false;
end;

procedure TvoieCalcDlg.EditNomChange(Sender: TObject);
begin
     AffichageGB.caption := 'Affichage'+EditUnite.text+')';
     ExpressionEdit.EditLabel.caption := EditNom.Text+'=';
     modif := true;
end;

procedure TvoieCalcDlg.HelpBtnClick(Sender: TObject);
begin
     Application.HelpContext(HELP_Voiecalculee)
end;

procedure TvoieCalcDlg.ListGrandeursClick(Sender: TObject);
var caracSuivant : char;
begin
     ClipBoard.Astext := ListGrandeurs.Items[ListGrandeurs.ItemIndex];
     ExpressionEdit.PasteFromClipboard;
     ExpressionEdit.setFocus;
     caracSuivant := ExpressionEdit.text[ExpressionEdit.SelStart+ExpressionEdit.SelLength+1];
     if charinset(caracSuivant,[',',')','*','+','-','/']) then
        SendVirtualKey(vk_right);
     ExpressionEdit.SelLength := 0;
     ExpressionEdit.setFocus;        
end;

procedure TvoieCalcDlg.EditNomKeyPress(Sender: TObject; var Key: Char);
begin
     if not caracNom(key) then key := #0
end;

procedure TvoieCalcDlg.ExpressionEditChange(Sender: TObject);
begin
     modif := true;
end;

procedure TvoieCalcDlg.EditMiniKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     if key=vk_down then editMaxi.setFocus;
end;

procedure TvoieCalcDlg.EditMaxiChange(Sender: TObject);
begin
     modif := true;
end;

procedure TvoieCalcDlg.EditMaxiKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     if key=vk_up then editMini.setFocus;
end;

procedure TvoieCalcDlg.EditMiniChange(Sender: TObject);
begin
     modif := true;
end;

procedure TvoieCalcDlg.EditMiniExit(Sender: TObject);
begin with voieCalculee do begin
     try
     mini := strToFloatWin(EditMini.text);
     maxi := strToFloatWin(EditMaxi.text);
     except
     end;
end end;

procedure TvoieCalcDlg.EditMaxiKeyPress(Sender: TObject; var Key: Char);
begin
   if key=crCR then EditMiniExit(sender)
end;

procedure TvoieCalcDlg.EditMiniKeyPress(Sender: TObject; var Key: Char);
begin
     if key=crCR then editMiniExit(sender)
end;

end.

