unit OrphyLabGraphe;

interface

uses windows, SysUtils, Graphics, Classes, Grids,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
   system.UITypes, system.Types,
   Math, maths, orphyLabUtil, uniteKer, orphyLabDef, orphyLabConst,
   Printers, clipbrd, inifiles, comctrls, parser10;

const
   IntSansSignif         = -1;                                              
   dimPoint: integer     = 2;
   scaleCurseur          = 10;
   LongNombreAxe         = 4;
   NbreGraduations       = 10;
   NbreGradMin           = 5;
   CouleurGrille: Tcolor = clGray;
   PasSortieBinaire      = 512e-6; // seconde
   Fleche: array[boolean] of char = (WideChar($2193),WideChar($2191));
   couleurCurseur : array[1..3] of Tcolor = (clBlue,clRed,clBlack);
   NbreAffMin = 16; // pour exploitation des donnes
   CouleurRepere: Tcolor = clBlue;


type
   Trepere = class
      Valeur: double;
      edit: Tedit;
      destructor Destroy;override;
   end;

   TmodeTrace=(mtAuto,mtLigne,mtMotif,mtMotifLigne);
   TAffichagePeriode = (aPeriode,aFrequence,aNone); // aPhase

// dessin
   TselectDessin = (sdNone, sdCadre, sdPoint1, sdPoint2, sdCentre);
   Tcurseur = (curSelect, curTexte, curLigne, curSynchro, curEchelle,
               curZero,cur1,cur2);
// fin dessin

   Tcourbe = class(Tvoie)
   public
      a, b:        double; { entier = rel*a+b }
      deltaAxe, exposant, pointDebut: double;
// deltaAxe = cart entre graduations sur les axes
      NbreChiffre: integer;
      courbeBottom, courbeTop: integer;
      LabelVoie,LabelVumetre:   Tlabel;
      GBvoie,GBVumetre:  TgroupBox;
      BtnVoie:     TbitBtn;
      BtnUI : TtoolButton;
      Yvisible : boolean;
      indexZero1,indexZero2 : integer;
      valeurMoy,valeurMax,valeurMin,valeurEff,valeurPeak : double;

// capteur
      positif : boolean;
      UnVolt,ZeroVolt: double;
      Aide: string;
      isLumicolor,isPhotocolor: boolean;
      tensionRef: double;
      oldCapsule,capsulePrecedente : Tcapsule;
      constructor Create(Anom: string; Aunite: string;
         Asignif: string; Anumero: integer);
      procedure setDelta(grandAxe: boolean; min, max: double);
      function FormatAxe(x: double): string;
      function XWindow(r: double): integer;
      function XMonde(xi: integer): double;
      procedure chercheMinmax(Nbre: integer; var min, max: double);
      Procedure getCapteurVolt(capteurCourant : Tcapteur);
      procedure load(Avoie: string; Fichier: tMemIniFile); virtual;
      procedure store(Avoie: string; Fichier: tMemIniFile); virtual;
   end;

     TlisteRepere = class(TList)
  protected
    function Get(Index: integer): TRepere;
    procedure Put(Index: integer; Item: TRepere);
  public
    function Add(Item: TRepere): integer;
    function Remove(Item: TRepere): integer;
    property Items[Index: integer]: TRepere Read Get Write Put; default;
    procedure Clear; override;
  end;


   TsortieBinaire = class(Tcourbe)
   private
      FnumeroSB: integer;
      procedure setNumeroSB(Value: integer);
   public
      DebutPulse, NbrePulse, LargeurPulse, DistancePulse: integer;
      Canvas: Tcanvas;
      vecteurX: vecteur;
      Ax, Bx, Ay, By: double;
      LimiteFenetre: Trect;
      NbrePoints: indiceVecteur;
      y0: double;
      procedure Draw;
      property NumeroSB: integer Read FnumeroSB Write setNumeroSB;
      procedure calcVecteur;
      function Ordre: AnsiString;
      constructor Create;
      function EtatSortie(t: double): integer;
   end;

   TvoieCalculee = class(Tcourbe)
   protected
      Parser:      Tparser;
      valeurEA: array[indiceVoie] of PParserFloat;
      function getExpression: string;
      procedure setExpression(Avalue: string);
   public
      property expression : String read getExpression write setExpression;
      procedure CalcValeur;
      function calibreVoie: string;
      procedure setMinMax;
      procedure RecopieC(N: integer);
      constructor Create;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
      destructor Destroy; override;
   end;

   TvoieCapteur = class(Tcourbe)
   public
      Function ValeurReal(V : word) : double; override;
      Function ValeurVolt(V : word) : double;
      constructor Create;
   end;

   Tdessin = class
      x1, y1, x2, y2: double;
      x1i, y1i, x2i, y2i: integer;
      cadre:       Trect;
      hauteur:     integer;
      avecCadre:   boolean;
      pen:         Tpen;
      isIdentification: boolean;
      isTexte:     boolean;
      isOpaque:    boolean;
      couleurFond: Tcolor;
      Texte:       TStringList;
      NumVoie:     integer;
      constructor Create;
      procedure Draw;
      procedure AffectePosition(x, y: integer; PosDessin: TselectDessin;
         Shift: TShiftState);
      function LitOption: boolean;
      destructor Destroy; override;
   end;

   TlisteDessin = class(TList)
   protected
      function Get(Index: integer): Tdessin;
      procedure Put(Index: integer; Item: Tdessin);
   public
      function Add(Item: Tdessin): integer;
      function Remove(Item: Tdessin): integer;
      property Items[Index: integer]: Tdessin Read Get Write Put; default;
      procedure Clear; override;
   end;

   Tgraphe = class
      ListeRepere:  TListeRepere;
      Canvas:           Tcanvas;
      traceLigne, traceMotif: boolean;
      NbreCourbe:       integer;
      codeErreur:       integer;
      avecAxeX, avecAxeY: boolean;
      grandAxeX, grandAxeY: boolean;
      voieOrphy:        array[indiceVoie] of Tcourbe;
      valeurSynchro:    integer;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      NbreInter, NbreAff, NbreCourant : integer;
      voieAbscisse,voieYprinc:  indiceVoie;
      modeTrace : TmodeTrace;
      quadrillage, AnalyseurLogique: boolean;
      montant,preTrig:  boolean;
      CapsuleStimul: boolean;
      impressionEnCours: boolean;
      Dessins:          TlisteDessin;
      DessinPeriode: Tdessin;
      VoieSynchro:      indiceVoie;
      VoieAnaMax : indiceVoie;
      VecteurCardiaque : vecteur;
      CapsuleCardiaque : boolean;
      loupeAuto : boolean;
      curseurs : array[1..2] of Tpoint;
      curseursActifs : boolean;
      affichagePeriode : TaffichagePeriode;
      avecFrequence:  boolean;
      nbrePages,premierePage : indicePage;
      oldNbre : array[indicePage] of integer;
      constructor Create;
      procedure Draw;
      procedure TraceCourbes;
      procedure TraceCurseurs;
      procedure VerifCurseurs;
      Procedure GetCurseurs(grid : TstringGrid);
      procedure TracePoint(voie: integer);
      procedure TraceReticule(voie: integer);
      procedure SetHauteurFonte(H: integer);
      procedure CourantVersAff;
      procedure setOrigineTemps;
      destructor Destroy; override;
      procedure TraceMotifs(xi, yi: integer; motif: Tmotif);
      procedure VersBitMap(NomFichier: string);
      procedure Print;
      Function isAxeY(x,y : integer) : indiceVoie;
      Function isZeroY(x,y : integer;voie : indiceVoie) : boolean;
      function changeEchelleY(ynew,yold : integer;voie : indiceVoie) : boolean;
      function changeZeroY(ynew,yold : integer;voie : indiceVoie) : boolean;
      procedure Segment(x1, y1, x2, y2: integer);
      procedure penDefault;
      procedure GetDessin(x, y: integer; var Adessin: Tdessin;
         var posDessin: TselectDessin);
      procedure RectangleGr(Arect: Trect);
      procedure LineGr(Arect: Trect);
      procedure AffecteCentreRect(x, y: integer; var r: Trect);
      procedure setSeuilSynchro(sync: double);
      function VoieSynchroChar : ansichar;
      procedure TraceRepere(repere: Trepere);
      procedure sauvePage;
   end;

var
   graphe:           Tgraphe;
   EntreeClavier:    Tcourbe;
   voieCalculee:     TvoieCalculee;
   voieCapteur6V:    TvoieCapteur;
   Frequence:        Tunite;
   Angle: Tunite;
   DureeBalayage, Deltat: double;
   ModeSynchro:      TModeSynchro;
   ModeAcq:          TModeAcq;
   EcartAuto:        integer;
   SortieBinaire:    TsortieBinaire;

implementation

uses lectText, options_Ligne;

var
   MaxGr, MinGr: integer;

function CouleurPale(couleur : Tcolor;indexPage: integer): Tcolor;
var
  delta : integer;
  R,G,B : integer;
begin
      if indexPage<0 then indexPage := maxPages-indexPage;
      indexPage := indexPage mod pasPage;
      inc(indexPage);
      delta := indexPage*64;
      B := getBvalue(couleur) + delta;
      if B>255 then B := 255;
      R := getRvalue(couleur) + delta;
      if R>255 then R := 255;
      G := getGvalue(couleur) + delta;
      if G>255 then G := 255;
      Result := RGB(R,G,B);
end;

function StylePage(indexPage: integer): TpenStyle;
begin
      if indexPage<0 then indexPage := maxPages-indexPage;
      result := TpenStyle(indexPage div pasPage);
end;


function Tcourbe.XMonde(xi: integer): double;
begin
   Result := (xi - B) / A;
end;

function Tcourbe.XWindow(r: double): integer;
begin
   if isNan(r) then
      Result := IntSansSignif
   else
      try
         Result := round(A * r + B)
      except
         Result := IntSansSignif
      end;
end;

procedure Tgraphe.Segment(x1, y1, x2, y2: integer);
begin
   Canvas.moveTo(x1, y1);
   Canvas.lineTo(x2, y2);
end;

procedure Tgraphe.TraceMotifs(xi, yi: integer; motif: Tmotif);
var
   PolyLigne: array[1..5] of Tpoint;
begin
   case motif of
      mCroix: begin
         Segment(xi - dimPoint, yi, xi + dimPoint, yi);
         Segment(xi, yi - dimPoint, xi, yi + dimPoint);
      end;
      mCroixDiag: begin
         segment(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
         segment(xi + dimPoint, yi - dimPoint, xi - dimPoint, yi + dimPoint);
      end;
      mCercle: Canvas.ellipse(xi - dimPoint, yi - dimPoint, xi +
            dimPoint + 1, yi + dimPoint + 1);
      mCarre: canvas.rectangle(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
      mLosange: begin
         PolyLigne[1] := Point(xi - dimPoint, yi);
         PolyLigne[2] := Point(xi, yi - dimPoint);
         PolyLigne[3] := Point(xi + dimPoint, yi);
         PolyLigne[4] := Point(xi, yi + dimPoint);
         PolyLigne[5] := Point(xi - dimPoint, yi);
         canvas.polyLine(PolyLigne);
      end;
   end;
end;

procedure Tgraphe.TracePoint(voie: integer);
var
   xi, yi: integer;
begin with voieOrphy[voie] do begin
      canvas.pen.color := couleur;
      canvas.brush.color := couleur;
      xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
         if dimPoint = 0
            then canvas.pixels[xi, yi] := couleur
            else traceMotifs(xi, yi, motif);
end end; // TracePoint

procedure Tgraphe.TraceReticule(voie: integer);
var
   xi, yi: integer;
begin with voieOrphy[voie] do begin
      if not Yvisible then exit;
      xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
         with canvas do begin
            pen.color := couleur;
            brush.color := couleur;
            pen.mode  := pmNotXor;
            pen.Width := 3;
            segment(xi - scaleCurseur, yi,xi + scaleCurseur, yi);
            segment(xi, yi - scaleCurseur,xi, yi + scaleCurseur);
            penDefault;
         end;
end end; // TraceReticule

procedure Tgraphe.SetHauteurFonte(H: integer);
var S : integer;
begin
   S := H * 72 div canvas.Font.PixelsPerInch;
   S := 2 * (S div 2);
   if S <= 8 then S := 8;
   canvas.font.Name := 'Calibri';
   if ImpressionEnCours
      then begin
         if (S>20) then S := 20;                     
      end
      else begin
         if  (S>16) then S := 16;
      end;
   canvas.font.size := S;
   HautCarac := canvas.TextHeight('A');
   LargCarac := canvas.TextWidth('A');
   Marge := largCarac div 2;
end;

procedure Tgraphe.Draw;

   procedure ChercheMonde;
   var
      voie: indiceVoie;
      N, hauteurCourbe, Nc: integer;
      maxLoc, MinLoc: double;
   begin
      for voie := voie0 to voieMax do begin
          voieOrphy[voie].Yvisible := false;
//          voieOrphy[voie].reticuleTrace := false;
      end;
      NbreCourbe := 0;
      for voie := voie0 to voieAnaMax do with voieOrphy[voie] do begin
          Yvisible := active and
                   (voie<>voieAbscisse) and
                   not isSynchro;
(*
          if Yvisible and (voie=voieU3) then
             Yvisible := not voieCapteur6V.active
                         or voieCapteur6V.avecTension;
*)
          if Yvisible then Inc(NbreCourbe);
      end;
      voieCalculee.Yvisible := voieCalculee.active;
      if voieCalculee.active then
         Inc(NbreCourbe);
      if AnalyseurLogique
         then N := NbreCourbe
         else N := 1;
      with limiteFenetre do begin
         avecAxeY := ( (bottom - top) > NbreGradMin * hautCarac * N );
         avecAxeX := (right - left) > NbreGradMin * 4 * largCarac;
         grandAxeY := (bottom - top) > NbreGraduations * HautCarac * N;
         grandAxeX := (right - left) > NbreGraduations * 4 * largCarac;
         MaxGr := 2 * right;
         MinGr := -right;
      end;
      limiteCourbe := limiteFenetre;
      Inc(limiteCourbe.left, Marge);
      Dec(limiteCourbe.bottom, Marge);
      if not (AnalyseurLogique) then
         Inc(limiteCourbe.top, Marge);
      if avecAxeY then begin
         Inc(limiteCourbe.left, LongNombreAxe * largCarac);
         if not (AnalyseurLogique) then
            Inc(limiteCourbe.top, hautCarac);
      end;
      if avecAxeX then
         Dec(limiteCourbe.bottom, hautCarac);
      if AnalyseurLogique then begin
         hauteurCourbe := (LimiteCourbe.bottom - LimiteCourbe.top) div N - marge;
         Nc := 0;
         for voie := voie0 to voieMax do
            with voieOrphy[voie] do
               if active and (voie <> voieAbscisse) then begin
                  CourbeTop := LimiteCourbe.top + Nc * hauteurCourbe +
                     succ(Nc) * marge;
                  CourbeBottom := CourbeTop + hauteurCourbe;
                  inc(Nc);
               end;
      end
      else
         for voie := voie0 to voieMax do
            with voieOrphy[voie] do
               if active and (voie <> voieAbscisse) then begin
                  CourbeTop := LimiteCourbe.top;
                  CourbeBottom := LimiteCourbe.bottom;
               end;
      with voieOrphy[voieAbscisse] do begin
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta(grandAxeX, mini, maxi);
      end;
      for voie := voie0 to voieAnaMax do
         with voieOrphy[voie] do
            if active and (voie <> voieAbscisse) then begin
               MaxLoc := Maxi;
               MinLoc := Mini;
               if utilisationEF=fFrequence then begin
                  if (maxi<maxiCapteur) then begin
                     chercheMinMax(NbreAff, MinLoc, MaxLoc);
                     if MaxLoc>Maxi then Maxi := MaxLoc;
                  end;
                  MaxLoc := Maxi;
                  MinLoc := 0;
                  if loupeAuto and (NbreAff > NbreAffMin)
                     then chercheMinMax(NbreAff, MinLoc, MaxLoc);
               end;
               if ((capsule=nil) or (Capsule.casParticulier<>cMagnum)) and
                  (utilisationEF<>fFrequence) then begin
                  if Maxi>MaxiCapteur then Maxi := MaxiCapteur;
                  if Mini<MiniCapteur then Mini := MiniCapteur;
                  if (maxi<maxiCapteur) or (mini>miniCapteur) then begin
                     chercheMinMax(NbreAff, MinLoc, MaxLoc);
                     if MaxLoc>Maxi then Maxi := MaxLoc;
                     if MinLoc<Mini then
                     if positif
                        then if MinLoc>0
                           then Mini := MinLoc
                           else Mini := 0
                        else Mini := MinLoc;
                  end;
                  MaxLoc := Maxi;
                  MinLoc := Mini;
                  if loupeAuto and (NbreAff > NbreAffMin)
                     then chercheMinMax(NbreAff, MinLoc, MaxLoc)
               end;
               if AnalyseurLogique and positif and (MinLoc=0) then
                  MinLoc := -MaxLoc/25;
               A := (CourbeTop - CourbeBottom) / (MaxLoc - MinLoc);
               B := CourbeBottom - MinLoc * A;
               setDelta(grandAxeY, MinLoc, MaxLoc);
      end;                                                 
      with voieCalculee do
         if active then begin
            A := (CourbeTop - CourbeBottom) / (Maxi - Mini);
            B := CourbeBottom - Mini * A;
            setDelta(grandAxeY, Mini, Maxi);
         end;
      with voieCapteur6V do
         if active then begin
            A := (CourbeTop - CourbeBottom) / (Maxi - Mini);
            B := CourbeBottom - Mini * A;
            setDelta(grandAxeY, Mini, Maxi);
         end;
   end; // ChercheMonde

procedure DrawAxis;// trac des axes
var xAxeY,yAxeX : integer;
    xNombreMax : integer;
    PointCourant : single;
    couleurAxe : TcolorRef;

Function YcorrectNombre(y : integer;m : indiceVoie) : boolean;
var limiteTop : integer;
begin with voieOrphy[m] do begin
     LimiteTop := CourbeTop+marge;
     if AnalyseurLogique then inc(LimiteTop,hautCarac);
     YcorrectNombre := ( y>(LimiteTop)) and
                       ( y<=(CourbeBottom-marge)) and
                       ( abs(y-yAxeX)>marge )
end end;

Function YcorrectGrad(y : integer;m : indiceVoie) : boolean;
begin with voieOrphy[m] do
     YcorrectGrad := (y>=CourbeTop) and (y<=CourbeBottom)
end;

Function XcorrectNombre(x : integer) : boolean;
begin
     XcorrectNombre := (x>(limiteCourbe.left+marge)) and
                       (x<xNombreMax) and
                       (abs(x-xAxeY)>marge )
end;

Function XcorrectGrad(x : integer) : boolean;
begin
     XcorrectGrad := (x>limiteCourbe.left) and (x<limiteCourbe.right)
end;

Procedure graduationX(x : integer);
begin
  if quadrillage
       then segment(x,limiteCourbe.top,x,limiteCourbe.bottom)
       else segment(x,yAxeX,x,yAxeX+marge);
end;

Procedure graduationY(y : integer);
begin
   if quadrillage
       then segment(limiteCourbe.left,y,limiteCourbe.right,y)
       else segment(xAxeY,y,xAxeY+marge,y);
end;

  procedure initAxeX;
  var x1 : integer;
  begin with voieOrphy[voieAbscisse] do begin
     PointCourant := PointDebut;
     x1 := XWindow(0);
     if (x1>limiteCourbe.left) and (x1<limiteCourbe.right) then begin
          canvas.pen.style := psDash;
          canvas.pen.width := 1;
          segment(x1,limiteCourbe.bottom,x1,limiteCourbe.top);
          penDefault;
     end;
     xNombreMax := limiteFenetre.right-largCarac*longNombreAxe div 2;
     with limiteFenetre do begin
          segment(left,yAxeX,right,yAxeX);
          { flches }
          segment(right-marge*3,yAxeX+marge,right,yAxeX);
          segment(right-marge*3,yAxeX-marge,right,yAxeX);
     end;
     SetTextAlign(canvas.handle,TA_center+TA_top);
  end end;{initAxeX}

  procedure traceAxeX;
  var xs : integer;
      NombreAff : string;
  begin with voieOrphy[voieAbscisse] do begin
     if Quadrillage then begin
        canvas.pen.style := psDot;
        canvas.pen.width := 1;
        canvas.pen.color := CouleurGrille;
     end;
     repeat
          xs := Xwindow(PointCourant);
        if XcorrectGrad(xs) then begin
               graduationX(xs);
               if XcorrectNombre(xs) then begin
          if abs(PointCourant)< (Maxi-Mini)/100 then pointCourant := 0;
                  NombreAff := formatAxe(pointCourant/exposant);
          canvas.TextOut(xs,yAxeX+marge,NombreAff);
             end;
          end;
        PointCourant := PointCourant+deltaAxe;
     until ( PointCourant>=Maxi );
     penDefault;
     SetTextAlign(canvas.handle,TA_left+TA_top);
     canvas.TextOut(limiteFenetre.left+marge,yAxeX+marge,nomAxe(exposant));
 end end;// traceAxeX

 procedure initAxeY(m : indiceVoie);
 var y1 : integer;
 begin with voieOrphy[m] do begin
    couleurAxe := couleur;
    canvas.font.Color := CouleurAxe;
    canvas.pen.Color := CouleurAxe;
    PointCourant := PointDebut;
    y1 := XWindow(0);
    if (y1>CourbeTop) and (y1<CourbeBottom)then begin
       canvas.pen.style := psDash;
       canvas.pen.width := 1;
       segment(limiteCourbe.left,y1,limiteCourbe.right,y1);
       penDefault;
   end;
   segment(xAxeY,CourbeBottom,xAxeY,CourbeTop);
         { flches }
   segment(xAxeY-marge,courbeTop+3*marge,xAxeY,courbeTop);
   segment(xAxeY+marge,courbeTop+3*marge,xAxeY,courbeTop);
   setTextAlign(canvas.handle,TA_left+TA_bottom);
 end end; // initAxeY

 procedure traceAxeY(m : indiceVoie);
 var ys : integer;
     nombreAff : string;
     yTitre : integer;
 begin with voieOrphy[m] do begin
      if Quadrillage then begin
         canvas.pen.style := psDot;
         canvas.pen.width := 1;
         canvas.pen.color := CouleurGrille;
      end;
      repeat
         ys := Xwindow(PointCourant);
// coordonne cran de marque
         if YcorrectGrad(ys,m) then begin
           graduationY(ys);
           if YcorrectNombre(ys,m) then begin
               if abs(pointCourant) < (Maxi-Mini)/1000 then pointCourant := 0;
               NombreAff := formatAxe(pointCourant/exposant);
               Canvas.TextOut(marge,ys+marge,NombreAff);
           end;
        end;
       pointCourant := pointCourant+deltaAxe;
     until ( pointCourant>=Maxi );
     penDefault;
     if analyseurLogique and (nbreCourbe>1)
        then yTitre := courbeTop
        else yTitre := limiteFenetre.Top+marge;
     SetTextAlign(canvas.handle,TA_left+TA_top);
     canvas.TextOut(limiteCourbe.left+largCarac,yTitre,nomAxe(exposant));
 end end;{traceAxeY}

procedure zeroX;
var x1 : integer;
begin with voieOrphy[voieAbscisse] do
   x1 := XWindow(0);
   if (x1>limiteCourbe.left) and (x1<limiteCourbe.right) then begin
        canvas.pen.style := psDot;
        canvas.pen.width := 1;
        segment(x1,limiteCourbe.bottom,x1,limiteCourbe.top);
        penDefault;
   end;
end;{zeroX}

 procedure zeroY(m : indiceVoie);
 var y1 : integer;
 begin with voieOrphy[m] do begin
     y1 := Xwindow(0);
     if (y1>=CourbeTop) and (y1<=CourbeBottom) then begin
          canvas.pen.style := psDot;
          canvas.pen.width := 1;
          segment(limiteCourbe.left,y1,limiteCourbe.right,y1);
          penDefault;
     end
end end;{zeroY}

procedure drawAxeAnalog;
var voie : indiceVoie;
begin
  if AnalyseurLogique
     then begin
         for voie := voie0 to voieMax do with voieOrphy[voie] do
          if (voieOrphy[voie].active) and
             (voie<>voieAbscisse) and
             not voieOrphy[voie].isSynchro then
                if avecAxeY then begin
                   initAxeY(voie);
                   traceAxeY(voie);
                end
                else zeroY(voie);
     end
     else if avecAxeY
        then begin
           initAxeY(voieYprinc);
           traceAxeY(voieYprinc);
        end
        else zeroY(voieYprinc);
end;// DrawAxeAnalog

procedure drawAxeFreq;
var voie : indiceVoie;
begin
     if AnalyseurLogique then begin
        for voie := voie0 to voieMax do with voieOrphy[voie] do
            if (voieOrphy[voie].active) and
               (voie<>voieAbscisse) then
                if avecAxeY then begin
                   initAxeY(voie);
                   traceAxeY(voie);
               end
               else zeroY(voie);
     end else if avecAxeY
        then begin
           initAxeY(voieYprinc);
           traceAxeY(voieYprinc);
        end
        else zeroY(voieYprinc);
end; // DrawAxeFreq

var voie : indiceVoie;
begin // DrawAxis
     couleurAxe := voieOrphy[voieAbscisse].couleur;
     canvas.font.Color := CouleurAxe;
     canvas.pen.Color := CouleurAxe;
     yAxeX := limiteCourbe.bottom;
     xAxeY := limiteCourbe.left;
     voieYprinc := voie0;
     for voie := voie0 to voieMax do with voieOrphy[voie] do
         if Yvisible then begin
             voieYprinc := voie;
             break;
         end;
     if (modeAcq=AcqTemps) and
        (modeSynchro=s_seuil) then voieYPrinc := voieSynchro;         
     if avecAxeX then begin
          initAxeX;
          traceAxeX;
          zeroX;
     end;
     if (modeAcq=AcqTemps) and avecFrequence
        then drawAxeFreq
        else drawAxeAnalog;
     SetTextAlign(canvas.handle,TA_left+TA_top);
end; // DrawAxis

Procedure CherchePeriode;
var voie,voieP : indiceVoie;

(*
Function GetPhase : double;
var i,quartPeriode,decalage,i0,i1 : integer;
    somme,sommeQuartPeriode : double;
begin
    Somme := 0;
    SommeQuartPeriode := 0;
    i0 := voieOrphy[voie].indexZero1;
    i1 := voieOrphy[voie].indexZero2;
    quartPeriode := (i1-i0) div 4;
    if quartPeriode>i0
       then decalage := +quartPeriode
       else decalage := -quartPeriode;
    if (i1+decalage>NbreAff) then begin
       result := 0;
       exit;
    end;
    for i := i0 to pred(i1) do begin
          Somme := Somme+
                 voieOrphy[voie].vecteurAff[i]*
                 voieOrphy[voieP].vecteurAff[i];
           SommeQuartPeriode := SommeQuartPeriode+
                   voieOrphy[voie].vecteurAff[i]*
                   voieOrphy[voieP].vecteurAff[i+decalage];
    end;
    result := atan2(SommeQuartperiode,Somme);
end; // getPhase
*)

var signe,periode : double;
    zz : double;
    i,N : integer;
    sommeEff,sommeMoy : double;
//    phase : double;
begin
for voie := voie0 to voieAnaMax do
    if voieOrphy[voie].active then break;
for voieP := succ(voie) to voieAnaMax do
    if voieOrphy[voieP].active then break;
with voieOrphy[voie] do begin
     dessinPeriode.pen.color := couleur;
     valeurMax := vecteurAff[0];
     valeurMin := vecteurAff[0];
     for i := 1 to pred(NbreAff) do begin
        zz := vecteurAff[i];
        if zz>valeurMax then valeurMax := zz;
        if zz<valeurMin then valeurMin := zz;
     end;
     valeurMoy := (valeurMax+valeurMin)/2;
     indexZero1 := 0;
     signe := vecteurAff[0]-valeurMoy;
     if signe=0 then signe := -1;
     while (indexZero1<(NbreAff div 2)) and
           ((vecteurAff[indexZero1]-valeurMoy)*signe>0) do inc(indexZero1);
// Premier passage par "zro" (vers le bas par ex.)
     signe := vecteurAff[indexZero1]-valeurMoy;
     if signe=0 then signe := -1;
     indexZero2 := indexZero1+4;
     if indexZero2>NbreAff then exit;
     repeat inc(indexZero2)
     until ((vecteurAff[indexZero2]-valeurMoy)*signe<0) or
           (indexZero2>=(NbreAff-4));
// On repasse par "zro" dans le sens oppos
     indexZero2 := indexZero2+4;
     if indexZero2>NbreAff then exit;
     repeat inc(indexZero2)
     until ((vecteurAff[indexZero2]-valeurMoy)*signe>0) or
           (indexZero2>=pred(NbreAff));
// On repasse par "zro" dans le mme sens 
     if (indexZero2-indexZero1)<8 then exit;
     SommeEff := 0;
     SommeMoy := 0;
     N := succ(indexZero2-indexZero1);
     for i := indexZero1 to indexZero2 do begin
        zz := vecteurAff[i];
        SommeEff := SommeEff+sqr(zz);
        SommeMoy := SommeMoy+zz;
     end;
     valeurEff := sqrt(SommeEff/N);
     valeurMoy := sommeMoy/N;
 //    if voieOrphy[voieP].active then phase := getPhase;
     periode := voieOrphy[voieT].vecteurAff[indexZero2]-
                voieOrphy[voieT].vecteurAff[indexZero1];
     if periode=0 then
          dessinPeriode.Texte[0] := '?'
     else case affichagePeriode of
 //         aNone : dessinPeriode.Texte[0] := '';
          aPeriode : dessinPeriode.Texte[0] := 'T='+voieOrphy[voieT].formatUnite(periode);
          aFrequence : dessinPeriode.Texte[0] := 'f='+frequence.formatUnite(1/periode);
 //         aPhase : dessinPeriode.Texte[0] := phi+'='+angle.formatUnite(phase);
     end;
     dessinPeriode.numVoie := voie;
     if affichagePeriode<>aNone then dessinPeriode.draw;
end; // with voieOrphy[voie]
end;

var
   i, j: integer;
   supprDessin: boolean;
begin  // draw
   with limiteFenetre do
        setHauteurFonte((bottom - top) div 30);
   penDefault;
   canvas.pen.color := voieOrphy[voieT].couleur;
   setBkMode(canvas.handle, transparent);
   chercheMonde;
   drawAxis;
   TraceCourbes;
   i := 0;
   while (i < Dessins.Count) do
      with dessins[i] do
         if isTexte then begin
            SupprDessin := not (isIdentification);
            for j := 0 to pred(texte.Count) do
               if length(texte[j]) > 0 then
                  SupprDessin := False;
            if SupprDessin then
               Dessins.remove(dessins[i])
            else
               Inc(i);
         end
         else
            Inc(i);
   for i := 0 to pred(Dessins.Count) do
       dessins[i].draw;
   if (affichagePeriode<>aNone) and
      (modeAcq=AcqTemps) and
      (NbreAff>NbreAffMin) then cherchePeriode;
   if curseursActifs then verifCurseurs;
   for i := 0 to pred(ListeRepere.Count) do
       TraceRepere(ListeRepere[i]);
end; // Tgraphe.draw

procedure Tgraphe.TraceCourbes;

   procedure TraceUneCourbe(voie: integer);

      procedure traceImpulsion;
      var
         i: indiceVecteur;
         xi, y0, y1: integer;
      begin
         penDefault;
         canvas.pen.color := clRed;
         y0 := voieOrphy[voieSynchro].XWindow(0);
         y1 := voieOrphy[voieSynchro].XWindow(20);
         for i := 0 to pred(NbreAff) do begin
            xi := voieOrphy[voieAbscisse].XWindow(vecteurCardiaque[i]);
            segment(xi, y0, xi, y1);
         end;{for i}
      end; // traceImpulsion

      procedure TracePoints;
      var
         i: indiceVecteur;
         xi, yi: integer;
         pixel: boolean;
      begin with voieOrphy[voie] do begin
            canvas.brush.color := couleur;
            canvas.brush.style := bsSolid;
            pixel := (dimPoint = 0);
            for i := 0 to pred(NbreAff) do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
                  if pixel
                     then canvas.pixels[xi, yi] := couleur
                     else TraceMotifs(xi, yi, motif);
            end;{i}
            canvas.brush.style := bsClear;
      end end; // tracePoints

      procedure traceLignes;
      var
         i: indiceVecteur;
         PointPrecedent: boolean;
         xi, yi: integer;
      begin with voieOrphy[voie] do begin
            PointPrecedent := False;
            for i := 0 to pred(NbreAff) do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi = IntSansSignif) or (yi = IntSansSignif) then
                  pointPrecedent := False
               else if PointPrecedent then
                  canvas.LineTo(xi, yi)
               else begin
                  canvas.MoveTo(xi, yi);
                  pointPrecedent := True;
               end;
            end;{for i}
      end end; // traceLignes

      procedure traceOldLignes(j : integer);
      var
         i: indiceVecteur;
         xi, yi: integer;
         vecteurX,vecteurY : vecteur;
      begin
            vecteurY := voieOrphy[voie].oldVecteur[j];
            vecteurX := voieOrphy[voieAbscisse].oldVecteur[j];
            canvas.pen.color := couleurPale(voieOrphy[voie].couleur,premierePage-j);
            canvas.pen.style := stylePage(premierePage-j);
            xi := voieOrphy[voieAbscisse].XWindow(vecteurX[0]);
            yi := voieOrphy[voie].XWindow(vecteurY[0]);
            canvas.MoveTo(xi, yi);
            for i := 1 to pred(oldNbre[j]) do begin
               xi := voieOrphy[voieAbscisse].XWindow(vecteurX[i]);
               yi := voieOrphy[voie].XWindow(vecteurY[i]);
               canvas.LineTo(xi, yi);
            end;{for i}
      end; // traceOldLignes

   var
      yi: integer;
      j : integer;
   begin
      if CapsuleStimul and (voie = voieSynchro) then
         with canvas do begin
            yi := voieOrphy[voieSynchro].XWindow(voieOrphy[voieSynchro].valeurCourante);
            pen.style := psDash;
            pen.Width := 1;
            pen.color := voieOrphy[voieSynchro].couleur;
            segment(limiteCourbe.left, yi, limiteCourbe.right, yi);
            penDefault;
         end;
//      if (voie=voieU3) and not voieCapteur6V.avecTension then exit;
      if (modeSynchro = s_front) and
         (modeAcq = AcqTemps) and
         (voie = voieSynchro) and
         voieOrphy[voie0].isSynchro then
         exit;
      if Remanence and
         (modeAcq=AcqTemps) and
         (nbrePages>0) and
         (premierePage>0) then begin
         for j := 1 to nbrePages do traceOldLignes(j);
         if NbreAff>0 then canvas.pen.Width := 3;
      end;
      if NbreAff = 0 then exit;
      canvas.pen.color := voieOrphy[voie].couleur;
      if traceLigne then
         TraceLignes;                                          
      if traceMotif then
         TracePoints;
      if CapsuleCardiaque and (voie = voieSynchro) then
         TraceImpulsion;
      canvas.pen.Width := 1;
   end; // TraceUneCourbe

   procedure TraceAuto;
   begin
    if (modeAcq in [AcqTemps]) then begin
          traceLigne := true;
          traceMotif := false;
    end;
    if (modeAcq in [AcqPoint,AcqClavier]) then begin
          traceLigne := false;
          traceMotif := true;
    end;
    end;

var
   voie: indiceVoie;
   yi: integer;
begin // traceCourbes
    if modeTrace=mtAuto
       then traceAuto
       else begin
          traceLigne := (modeTrace=mtLigne) or (modeTrace=mtMotifLigne);
          traceMotif := (modeTrace=mtMotif) or (modeTrace=mtMotifLigne);
       end;
// AcqMagnum, AcqRadar, AcqTempsLong ?
   for voie := voie0 to voieMax do
      if voieOrphy[voie].active and
        (voie <> voieAbscisse) then
         traceUneCourbe(voie);
   if (modeAcq=AcqTemps) and (modeSynchro = s_seuil) then begin
      with voieOrphy[voieSynchro] do
           yi := XWindow(valeurReal(valeurSynchro));
      canvas.pen.style := psDashDot;
      canvas.pen.Width := 1;
      canvas.pen.color := voieOrphy[voieSynchro].couleur;
      canvas.font.Name := 'Calibri';
      canvas.font.color := voieOrphy[voieSynchro].couleur;
      segment(limiteCourbe.left - 16, yi, limiteCourbe.right, yi);
      canvas.brush.style := bsClear;
      canvas.TextOut(LimiteCourbe.left - 16, yi - 8, Fleche[montant]);
      penDefault;
   end;
end; // traceCourbes

destructor Tgraphe.Destroy;
var
   i: indiceVoie;
begin
   for i := voieMin to VoieMax do
      voieOrphy[i].Free;
   dessins.Clear;
   dessins.Free;
   dessinPeriode.Free;
   sortieBinaire.Free;
   ListeRepere.Free;
   inherited Destroy;
end;

constructor Tgraphe.Create;
var
   voie: indiceVoie;
begin
   inherited Create;
   ListeRepere := TListeRepere.Create;
   traceLigne := True;
   traceMotif := False;
   montant := True;
   pretrig := false;
   modeAcq := AcqTemps;
   modeSynchro := s_clavier;
   voieSynchro := voie0;
   voieOrphy[voieT] := Tcourbe.Create('t', 's', trDuree, voieT);
   EntreeClavier := Tcourbe.Create('X', '', '', voieClavier);
   voieOrphy[voieClavier] := EntreeClavier;
   voieCalculee := TvoieCalculee.Create;
   voieOrphy[voieCalc] := voieCalculee;
   voieCapteur6V := TvoieCapteur.Create;
   voieOrphy[voieCapteur] := voieCapteur6V;
   voieOrphy[voieCapteur].numeroOrphy := 11;   
   for voie := voie0 to voieMicroMax do
      voieOrphy[voie] := Tcourbe.Create('U' + IntToStr(voie), 'V', '', voie);
   for voie := voieI1 to voieI3 do begin
      voieOrphy[voie] := Tcourbe.Create('I' + IntToStr(voie-voieI1+1), 'A', '', voie);
      voieOrphy[voie].numeroOrphy := 12+(voie-voieI1);
      voieOrphy[voie].setMinMax(-0.25,0.25);
      voieOrphy[voie].capteur := 'Ampremtre';
   end;
   for voie := voieU1 to voieU3 do begin
      voieOrphy[voie] := Tcourbe.Create('U' + IntToStr(voie-voieU1+1), 'V', '', voie);
      voieOrphy[voie].numeroOrphy := 8+(voie-voieU1);
      voieOrphy[voie].setMinMax(-25,25);
      voieOrphy[voie].capteur := 'Voltmtre';
   end;
   for voie := voie0 to voieLabMax do
       voieOrphy[voie].active := false;
   AnalyseurLogique := False;
   Quadrillage  := False;
   voieAbscisse := voieT;
   SortieBinaire := TsortieBinaire.Create;
   for voie := voieMin to voieMax do
      with voieOrphy[voie] do
         if couleur = clWindow then
            couleur := clActiveCaption;
   CapsuleStimul := False;
   dessins := TlisteDessin.Create;
   dessinPeriode := Tdessin.create;
   with dessinPeriode do begin
      hauteur := 8;
      avecCadre := false;
      isIdentification := false;
      isTexte := true;
      isOpaque := true;
      couleurFond := clYellow;
      texte.add('priode');
   end;
end;

constructor Tcourbe.Create(Anom: string; Aunite: string;
   Asignif: string; Anumero: integer);
begin
   inherited Create(Anumero);
   active := False;
   A := 1;
   B := 0;
   nom := Anom;
   NomUnite := Aunite;
   signif := Asignif;
   deltaAxe := 1;
   exposant := 1;
   tensionRef := tensionRefLumicolor;
   oldCapsule := nil;
   capsulePrecedente := nil;
end;

procedure Tcourbe.SetDelta(grandAxe: boolean; min, max: double);
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   AbsMax, AbsMin, largeur: double;
   Exponent, index: integer;
   N: integer;
begin
   AbsMax := abs(max);
   AbsMin := abs(min);
   if AbsMax < AbsMin then
      AbsMax := AbsMin;
   Exponent := Plancher(Log(Max - Min));
   largeur := (max - min) * dix(-exponent);
   index := 0;
   if grandAxe
      then while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
      else while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * dix(exponent);
   NbreChiffre := Plafond(Log(AbsMax)) - Plancher(Log(DeltaAxe));
   PointDebut := Plafond(min / deltaAxe) * deltaAxe;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
// la division euclidienne des informaticiens <> div des maths !
   if nomUnite='tr/min' then index := 0;
   Exposant := Dix(3 * Index);
   N := Plafond(Log(AbsMax / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) then
      NbreChiffre := longNombreAxe;
end;

function Tcourbe.FormatAxe(x: double): string;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, NbreChiffre, 0);
end;

procedure Tgraphe.CourantVersAff;

   procedure RemoveGlitcheRadar;
   var
      j: integer;
      yg, dy: vecteur;
      ecart : double;
   begin
   ecart := 10 * coeffRadarTemps;
   if radar_position then ecart := ecart*vitesseSon;
   with voieOrphy[voie0] do begin
         yg[0] := vecteurAff[0];
         for j := 1 to pred(NbreAff) do begin
            dY[j] := vecteurAff[j] - vecteurAff[j - 1];
            yg[j] := vecteurAff[j];
         end;
         for j := 1 to (NbreAff - 2) do begin
            if (dy[j] * dy[j + 1] < 0) and (abs(dy[j]) > ecart) and
               (abs(dy[j + 1]) > ecart) then begin
               Yg[j] := (yg[j + 1] + yg[j - 1]) / 2;
               dy[j] := yg[j] - yg[j - 1];
               dy[j + 1] := yg[j + 1] - yg[j];
            end;
         end;
         if (abs(dy[NbreAff - 1]) > ecart) then
            Yg[NbreAff - 1] := Yg[NbreAff - 2] + dY[NbreAff - 2];
         if (abs(dy[1]) > ecart) then
            Yg[0] := Yg[1] - dY[2];
         for j := 0 to pred(NbreAff) do
            vecteurAff[j] := yg[j];
   end end; // RemoveGlitche

var
   i: integer;
   voie: indiceVoie;
begin
   for voie := voieMin to voieMax do
      with voieOrphy[voie] do
         for i := 0 to pred(NbreCourant) do
             vecteurAff[i] := vecteurCourant[i];
   NbreAff := NbreCourant;
   if capsuleStimul then
      with voieOrphy[voieSynchro] do begin
         valeurAff := 0;
         for i := 0 to pred(NbreCourant) do
            valeurAff := valeurAff + vecteurAff[i];
         valeurAff := valeurAff / nbreCourant;
         for i := 0 to pred(NbreCourant) do
            vecteurAff[i] := valeurAff * sortieBinaire.EtatSortie(
               voieOrphy[voieT].vecteurAff[i]);
      end;
   if modeAcq = AcqMagnum then with voieOrphy[voieT] do begin
       for i := 0 to pred(NbreAff) do
           vecteurAff[i] := vecteurCourant[i] - vecteurCourant[0];
       if (NbreAff>8) and (vecteurAff[pred(NbreAff)]>maxi) then begin
             maxi := vecteurAff[pred(NbreAff)];
             maxiCapteur := maxi;
       end;
       exit;
   end;
   if modeAcq = AcqRadar then removeGlitcheRadar;
   voieCalculee.recopieC(NbreAff);
end; // CourantVersAff

procedure Tgraphe.SauvePage;
var
   i: integer;
   voie: indiceVoie;
begin
   if NbreAff<>NbreInter then exit;
   inc(premierePage);
   if premierePage>maxPages then premierePage := 1;
   for voie := voieMin to voieMax do
      with voieOrphy[voie] do
         for i := 0 to pred(NbreAff) do
             oldVecteur[premierePage,i] := vecteurAff[i];
   oldNbre[PremierePage] := NbreAff;
   if nbrePages<maxPages then inc(NbrePages);
end; // CourantVersAff

constructor TsortieBinaire.Create;
begin
   inherited Create('SB', '', '', 0);
   debutPulse := 1;
   NbrePulse := 2;
   LargeurPulse := 1;
   DistancePulse := 20;
   FnumeroSB := 0;
   Mini := -0.1;
   Maxi := +1.1;
   MiniCapteur := Mini;
   MaxiCapteur := Maxi;
   calcVecteur;
end;


function TsortieBinaire.etatSortie(t: double): integer;
var
   i: integer;
begin
   t := t / PasSortieBinaire;
   Result := 0;
   for i := 0 to pred(nbrePulse) do
      if (t > i * DistancePulse + debutPulse) and
         (t < (i * DistancePulse + LargeurPulse + debutPulse)) then begin
         Result := 1;
         break;
      end;
end;

procedure TsortieBinaire.setNumeroSB(Value: integer);
begin
   if Value <= 3 then
      FnumeroSB := Value;
end;

procedure TsortieBinaire.CalcVecteur;
var
   j, offset: integer;
begin
   NbrePoints  := 4 * NbrePulse + 2;
   vecteurAff[0] := 0;
   vecteurX[0] := 0;
   for j := 0 to pred(NbrePulse) do begin
      offset := j * 4;
      vecteurAff[offset + 1] := 0;
      vecteurX[offset + 1] := (debutPulse + j * distancePulse) * PasSortieBinaire;
      vecteurAff[offset + 2] := 1;
      vecteurX[offset + 2] := vecteurX[offset + 1];
      vecteurAff[offset + 3] := 1;
      vecteurX[offset + 3] := vecteurX[offset + 1] + largeurPulse * PasSortieBinaire;
      vecteurAff[offset + 4] := 0;
      vecteurX[offset + 4] := vecteurX[offset + 3];
   end;
   vecteurAff[pred(NbrePoints)] := 0;
   vecteurX[pred(NbrePoints)] := vecteurX[NbrePoints - 2] * 2;
   if DureeBalayage > vecteurX[pred(NbrePoints)] then
      vecteurX[pred(NbrePoints)] := DureeBalayage;
end;

procedure TsortieBinaire.Draw;
var
   maxiX, miniX: double;
   i: indiceVecteur;
   xi, yi: integer;
begin
   canvas.pen.color := clBlue;
   miniX := vecteurX[0];
   maxiX := vecteurX[pred(NbrePoints)];
   AX := (limiteFenetre.right - limiteFenetre.left) / (MaxiX - MiniX);
   BX := limiteFenetre.left - MiniX * AX;
   AY := (limiteFenetre.top - limiteFenetre.bottom) / (Maxi - Mini);
   BY := limiteFenetre.Bottom - Mini * AY;
   xi := round(AX * vecteurX[0] + BX);
   yi := round(AY * vecteurAff[0] + BY);
   canvas.MoveTo(xi, yi);
   for i := 0 to pred(NbrePoints) do begin
      yi := round(AY * vecteurAff[i] + BY);
      canvas.LineTo(xi, yi);
      xi := round(AX * vecteurX[i] + BX);
      canvas.LineTo(xi, yi);
   end; // for i
end;

function TsortieBinaire.Ordre: ansiString;
begin
   Result := ZgeneAcqui + ansiChar(NumeroSB) + codeValeur2(NbrePulse) +
      codeValeur2(LargeurPulse) + codeValeur2(DistancePulse) +
      codeValeur2(DebutPulse);
end;

procedure Tgraphe.SetOrigineTemps;
// pb du pretrig alatoire
var
   i,imax,j: integer;
   y1, y2, t1, t2, t0: double;
   valeurSeuil, valeurLimite: double;
   frontAtteint : boolean;
begin
   ValeurSeuil := voieOrphy[voieSynchro].valeurReal(valeurSynchro);
   if pretrig then begin
      i := 0;
      imax := NbreInter div 4;
      with voieOrphy[voieSynchro] do begin
         // on attend d'tre en dessous
         if montant
            then frontAtteint := vecteurAff[i] <= valeurSeuil
            else frontAtteint := vecteurAff[i] >= valeurseuil;
         while not frontAtteint and (i<imax) do begin
            inc(i);
            if montant
              then frontAtteint := vecteurAff[i] <= valeurSeuil
              else frontAtteint := vecteurAff[i] >= valeurseuil;
         end;
         // on attend la traverse
         if montant
             then frontAtteint := vecteurAff[i] >= valeurseuil
             else frontAtteint := vecteurAff[i] <= valeurseuil;
         while not frontAtteint and (i<imax) do begin
            inc(i);
            if montant
               then frontAtteint := vecteurAff[i] >= valeurseuil
               else frontAtteint := vecteurAff[i] <= valeurseuil;
         end;
      end;
      if i>=imax then i := 0;
      with voieOrphy[voieT] do begin
         for j := 0 to NbreInter do
            vecteurAff[j] := deltat * (j - i);
         mini := vecteurAff[0];
         maxi := vecteurAff[NbreInter];
      end;
   end
   else begin
   ValeurLimite := (voieOrphy[voieSynchro].maxiCapteur - voieOrphy[voieSynchro].miniCapteur) / 50;
   if valeurSeuil < valeurLimite then begin
      // on considre que c'est zro et on met t=0 pour V=0
      y1 := voieOrphy[voieSynchro].vecteurAff[0];
      y2 := voieOrphy[voieSynchro].vecteurAff[1];
      t1 := voieOrphy[voieT].vecteurAff[0];
      t2 := voieOrphy[voieT].vecteurAff[1];
      if y1=y2 then exit;
      try
         t0 := t1 - y1 * (t2 - t1) / (y2 - y1);
      except
         exit;
      end;
      if (t0 > 0) or (abs(t0) > 4 * deltat) then
         exit;
      for i := 0 to MaxVecteur do
          voieOrphy[voieT].vecteurAff[i] := voieOrphy[voieT].vecteurAff[i] - t0;
   end;
   end;
end;

procedure TGraphe.Print;
const
   MargeImprimante = 21;
   MargeImprimantePaysage = 30; // soit 1 cm
var
   margeI: integer;
begin
   printer.beginDoc;
   limiteFenetre := Printer.Canvas.ClipRect;
   if printer.orientation = poPortrait
      then margeI := Printer.PageWidth div margeImprimante
      else margeI := Printer.PageWidth div margeImprimantePaysage;
   with limiteFenetre do begin
         top  := top + margeI;
         bottom := bottom - margeI;
         right := right - margeI;
         left := left + margeI;
   end;
   canvas := Printer.Canvas;
   ImpressionEnCours := True;
   canvas.Font.Name := 'Calibri';
   canvas.Font.Color := clBlack;
   draw;
   ImpressionEnCours := False;
   printer.endDoc;
end;

procedure TGraphe.VersBitMap(NomFichier: string);
const
   XmaxBitmap: integer = 640;
   YmaxBitmap: integer = 400; {16/10}
var
   AbitMap: TbitMap;
begin
   AbitMap := TbitMap.Create;
   AbitMap.Height := YmaxBitmap;
   AbitMap.Width := XmaxBitmap;
   AbitMap.pixelFormat := pf24bit;
   setRect(limiteFenetre, 0, 0, XmaxBitmap, YmaxBitmap);
   canvas := AbitMap.Canvas;            
   Draw;
   if NomFichier = ''
      then ClipBoard.Assign(AbitMap)
      else AbitMap.saveToFile(NomFichier);
   AbitMap.Free;
end;

procedure Tcourbe.chercheMinmax(Nbre: integer; var min, max: double);
var
   i: integer;
   marge: double;
begin
   Min := vecteurAff[0];
   Max := Min;
   for i := 1 to pred(Nbre) do
      if vecteurAff[i] < Min
         then Min := vecteurAff[i]
         else if vecteurAff[i] > Max
            then Max := vecteurAff[i];
   marge := (maxiCapteur - miniCapteur) / 50;
   min := min - marge;
   if min < miniCapteur then
      min := miniCapteur;
   max := max + marge;
   if max > maxiCapteur then
      max := maxiCapteur;
end;

procedure Tgraphe.PenDefault;
begin
   canvas.pen.mode  := pmCopy;
   canvas.pen.style := psSolid;
   if impressionEnCours
      then canvas.pen.Width := 3
      else canvas.pen.Width := 1;
end;

constructor TvoieCalculee.Create;
begin
   inherited Create('X', '', '', voieCalc);
   Parser := Tparser.create(nil);
end;

destructor TvoieCalculee.Destroy;
begin
   inherited Destroy;
   Parser.free;
end;

constructor TvoieCapteur.Create;
begin
   inherited Create('Y', '', '', voieCapteur);
   numeroOrphy := 11;
end;

function TvoieCalculee.CalibreVoie: string;
begin
   Result := '';
end;

procedure TvoieCalculee.CalcValeur;
var
   v: indiceVoie;
begin
      if not active then exit;
      for v := voie0 to graphe.voieAnaMax do
          if graphe.voieOrphy[v].active then
              Parser.SetVariable(graphe.voieOrphy[v].nom,
                                 graphe.voieOrphy[v].valeurCourante);
      try
         valeurCourante := Parser.Value;
      except
         valeurCourante := Nan;
      end;
      valeurAff := valeurCourante;
end;

procedure TvoieCalculee.RecopieC(N: integer);
var
   v: indiceVoie;
   i: integer;
begin
   if not active then exit;
   for v := voie0 to graphe.voieAnaMax do
       if graphe.voieOrphy[v].active then
          valeurEA[v] := Parser.SetVariable(graphe.voieOrphy[v].nom,
                                            graphe.voieOrphy[v].vecteurAff[0]);
   for i := 1 to Pred(N) do begin
      for v := voie0 to graphe.voieAnaMax do
          if graphe.voieOrphy[v].active then
             valeurEA[v]^ := graphe.voieOrphy[v].vecteurAff[i];
      try
         vecteurAff[i] := Parser.Value;
      except
         vecteurAff[i] := Nan;
      end;
   end; // for i
end;

procedure TvoieCalculee.Load(Avoie: string; fichier: tMemIniFile);
begin
    inherited load(Avoie,fichier);
    expression := fichier.ReadString(Avoie, 'Expression', '');
    nom := fichier.ReadString(Avoie, 'Symb', 'X');
    setMinMax;
end;

function TvoieCalculee.getExpression: string;
begin
     Result := parser.expression
end;

procedure TvoieCalculee.setMinMax;
var
   maxmax: double;
   exponent: integer;
   unite: coefUnite;
begin
   VerifMinMaxReal(mini, maxi);
   MaxMax := abs(maxi);
   if MaxMax < abs(mini) then
      MaxMax := abs(mini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0
      then Exponent := (Exponent div 3) - 1
      else Exponent := Exponent div 3;
   // la division euclidienne des informaticiens <> div des maths !
   unite := coefUnite(exponent + Ord(nulle));
   nomUniteAff := nomAff(puissanceUnite[unite]);
   NbreAvantVirgule := Plancher(Log(MaxMax/ExposantAff))+2;
   WithVirgule := NbreAvantVirgule<4;
   maxiCapteur := maxi;
   miniCapteur := mini;
end; // VoieCalculee.setMinMax

procedure Tdessin.Draw;
const
   marge = 3;
var
   Haut, Larg: integer;
   x, y, h: integer;
   i: integer;
   sauveDimPoint: integer;
   xt,yt : array[0..3] of integer;
begin // Tdessin.Draw 
   graphe.canvas.pen.style := pen.style;
   graphe.canvas.pen.color := pen.color;
   if isOpaque
      then begin
          graphe.canvas.brush.color := couleurFond;
          graphe.canvas.brush.style := bsSolid;
      end
      else begin
          graphe.canvas.brush.color := clWindow;
          graphe.canvas.brush.style := bsClear;
      end;
   x2i := graphe.voieOrphy[graphe.VoieAbscisse].Xwindow(x2);
   y2i := graphe.voieOrphy[NumVoie].Xwindow(y2);
   if isTexte then begin //  Vrifier texte visible
      with graphe.LimiteCourbe do begin
         if x2i > right then
            x2i := (4 * right + left) div 5;
         if x2i < left then
            x2i := (4 * left + right) div 5;
         if y2i > bottom then
            y2i := (4 * bottom + top) div 5;
      end;
      with graphe.LimiteFenetre do
         if y2i < top then
            y2i := (4 * top + bottom) div 5;
      graphe.canvas.font.color := pen.color;
      with graphe.limiteFenetre do
         h := (right - left + bottom - top) div 2;
      h := round(h * (hauteur / 100.0));
      graphe.setHauteurFonte(h);
      cadre := rect(x2i, y2i, x2i, y2i);
      i := 0;
      if isIdentification then begin
         texte.Clear;
         with graphe.voieOrphy[numVoie] do begin
            pen.color := couleur;
            texte.add(nom + '=' + signif);
         end;
      end;
      while i < texte.Count do
         if texte[i] = ''
            then texte.Delete(i)
            else Inc(i);
      for i := 0 to pred(texte.Count) do begin
         Larg := graphe.canvas.textWidth(texte[i]);
         Haut := graphe.canvas.textHeight(texte[i]);
         x := x2i - Larg div 2;
         if x < graphe.limiteCourbe.left then
            x := graphe.limiteCourbe.left;
         if x < (graphe.limiteFenetre.left + 2 * h) then
            x := graphe.limiteFenetre.left + 2 * h;
         if (x - marge) < cadre.left then
            cadre.left := x - marge;
         if (x + larg + marge) > cadre.right then
            cadre.right := x + larg + marge;
         y := y2i + i * Haut;
         cadre.bottom := y + haut;
         xt[i] := x;
         yt[i] := y;
         graphe.canvas.TextOut(x, y, texte[i]);
      end; // for i
      if isIdentification and graphe.TraceMotif then begin
         sauveDimPoint := dimPoint;
         dimPoint := h div 2;
         graphe.traceMotifs(cadre.left - h,
            (cadre.top + cadre.Bottom) div 2,
            graphe.voieOrphy[numVoie].motif);
         dimPoint := sauveDimPoint;
      end;
      if avecCadre then begin
         with cadre do
            graphe.canvas.Rectangle(cadre);
         for i := 0 to pred(texte.Count) do
            graphe.canvas.TextOut(xt[i], yt[i], texte[i]);
      end;
   end
   else begin
      x1i := graphe.voieOrphy[graphe.VoieAbscisse].Xwindow(x1);
      y1i := graphe.voieOrphy[NumVoie].Xwindow(y1);
      graphe.segment(x1i, y1i, x2i, y2i);
   end;
   graphe.canvas.brush.style := bsClear;
end; // Tdessin.Draw

destructor Tdessin.Destroy;
begin
   Texte.Free;
   Pen.Free;
   inherited Destroy;
end;

constructor Tdessin.Create;
begin
   inherited Create;
   x1  := 0;
   y1  := 0;
   x2  := 0;
   y2  := 0;
   x1i := 0;
   y1i := 0;
   x2i := 0;
   y2i := 0;
   pen := Tpen.Create;
   pen.color := clBlue;
   hauteur := 3;
   isTexte := True;
   Texte := TStringList.Create;
   isIdentification := False;
end;

procedure Tdessin.AffectePosition(x, y: integer; posDessin: TselectDessin;
   Shift: TShiftState);

   procedure Affecte1;
   begin with graphe do begin
         x1i := x;
         y1i := y;
         x1  := voieOrphy[VoieAbscisse].Xmonde(x1i);
         y1  := voieOrphy[numVoie].Xmonde(y1i);
      end;
   end;

   procedure Affecte2;
   begin with graphe do begin
         x2i := x;
         y2i := y;
         x2  := voieOrphy[VoieAbscisse].Xmonde(x2i);
         y2  := voieOrphy[numVoie].Xmonde(y2i);
   end end;

var
   dx, dy: integer;
begin
   if not isTexte and
     ((abs(x1i - x2i) + abs(y1i - y2i)) < 3) then begin
      x2i := x1i + 2;
      y2i := y1i + 2;
   end;
   case posDessin of
      sdCadre: Affecte2;
      sdPoint2: begin
         if not isTexte and (ssShift in Shift) then
            if abs(x - x1i) > abs(y - y1i) then
               y := y1i
            else
               x := x1i;
         Affecte2;
      end;
      sdPoint1: begin
         if not isTexte and (ssShift in Shift) then
            if abs(x - x2i) > abs(y - y2i) then
               y := y2i
            else
               x := x2i;
         Affecte1;
      end;
      sdCentre: begin
         dx  := x - ((x1i + x2i) div 2);
         dy  := y - ((y1i + y2i) div 2);
         x1i := x1i + dx;
         x2i := x2i + dx;
         y1i := y1i + dy;
         y2i := y2i + dy;
         with graphe do begin
            x2 := voieOrphy[VoieAbscisse].Xmonde(x2i);
            y2 := voieOrphy[numVoie].Xmonde(y2i);
            x1 := voieOrphy[VoieAbscisse].Xmonde(x1i);
            y1 := voieOrphy[numVoie].Xmonde(y1i);
         end;
      end;
      sdNone: ;
   end;
end;

function Tdessin.LitOption: boolean;
begin
   if isTexte then begin
      if LectureTexteDlg = nil then
         Application.createForm(TLectureTexteDlg, LectureTexteDlg);
      lectureTexteDlg.DessinLoc := self;
      litOption := lectureTexteDlg.showModal = mrOk;
   end
   else begin
      if OptionLigneDlg = nil then
         Application.createForm(TOptionLigneDlg, OptionLigneDlg);
      OptionLigneDlg.DessinLoc := self;
      litOption := OptionLigneDlg.showModal = mrOk;
   end;
end;

function TlisteDessin.Add(Item: Tdessin): integer;
begin
   Result := inherited Add(Item);
end;

function TlisteDessin.Get(Index: integer): Tdessin;
begin
   Result := Tdessin(inherited Get(Index));
end;

procedure TlisteDessin.Put(Index: integer; Item: Tdessin);
begin
   inherited Put(Index, Item);
end;

function TlisteDessin.Remove(Item: Tdessin): integer;
begin
   Result := inherited Remove(Item);
   Item.Free;
end;

procedure TlisteDessin.Clear;
var
   i: integer;
begin
   for i := 0 to pred(Count) do
      items[i].Free;
   inherited Clear;
end;

procedure TGraphe.GetDessin(x, y: integer; var Adessin: Tdessin;
   var posDessin: TselectDessin);
var longueur: integer;

   function distance: integer;
   var
      a, b, c: integer;
   begin with Adessin do begin
         A := y1i - y2i;
         B := x2i - x1i;
         C := y2i * x1i - y1i * x2i;
         Longueur := round(sqrt(A * A + B * B));
         Result := abs(A * x + B * y + C) div longueur;
      end;
   end;

var
   i: integer;
   multX, multY: integer;
   d1, d2, d3, dmin: integer;
begin
   posDessin := sdNone;
   Adessin := nil;
   for i := 0 to pred(dessins.Count) do begin
      Adessin := dessins[i];
      with Adessin do
         if isTexte then begin
            with cadre do begin
               MultX := (x - left) * (x - right);
               MultY := (y - top) * (y - bottom);
            end;
            if (MultX <= 0) and (MultY <= 0) then
               posDessin := sdCadre;
         end
         else { ligne } if distance < 8 then begin
            d1 := abs(x - x1i) + abs(y - y1i);
            d2 := abs(x - x2i) + abs(y - y2i);
            d3 := abs(x - ((x2i + x1i) div 2)) + abs(y - ((y2i + y1i) div 2));
            dmin := longueur div 3;
            if d1 < dmin then
               posDessin := sdPoint1;
            if d2 < dmin then
               posDessin := sdPoint2;
            if d3 < dmin then
               posDessin := sdCentre;
         end; // ligne distance<16
      if posDessin <> sdNone then
         break;
   end; {i}
   if (posDessin = sdNone) and
      (affichagePeriode<>aNone) and
      (modeAcq=AcqTemps) then
      with dessinPeriode do begin
            with cadre do begin
               MultX := (x - left) * (x - right);
               MultY := (y - top) * (y - bottom);
            end;
            if (MultX <= 0) and (MultY <= 0) then begin
               posDessin := sdCadre;
               Adessin := dessinPeriode;
            end;
      end;
   if posDessin = sdNone then
      Adessin := nil;
end;

procedure Tgraphe.RectangleGr(Arect: Trect);
begin
   canvas.pen.mode := pmNotXor;
   canvas.Rectangle(Arect);
   penDefault;
end;

procedure Tgraphe.LineGr(Arect: Trect);
begin
   canvas.pen.mode := pmNotXor;
   segment(Arect.left, Arect.top, Arect.right, Arect.bottom);
   penDefault;
end;

procedure TGraphe.AffecteCentreRect(x, y: integer; var r: Trect);
var
   maxi: integer;
begin with r do begin
      maxi := limiteFenetre.right;
      x := x + (right - left) div 2;
      if x > maxi then
         x := maxi;
      maxi := limiteFenetre.left;
      x := x - (right - left);
      if x < maxi then
         x := maxi;
      maxi := limiteFenetre.bottom;
      y := y + (bottom - top) div 2;
      if y > maxi then
         y := maxi;
      maxi := limiteFenetre.top;
      y := y - (bottom - top);
      if y < maxi then
         y := maxi;
      r := rect(x, y, x + right - left, y + bottom - top);
end end;

procedure Tgraphe.setSeuilSynchro(sync: double);
begin with voieOrphy[voieSynchro] do begin
      valeurSynchro := valeurInt(sync);
      if valeurSynchro > 4000 then
         valeurSynchro := 4000;
      if valeurSynchro < 96 then
         valeurSynchro := 96;
end end;

function Tgraphe.VoieSynchroChar : ansichar;
var index : integer;
    voie : indiceVoie;
begin
     index := 0;
     if voieSynchro>voie0 then   // pour le compilateur !
        for voie := voie0 to pred(voieSynchro) do
            if voieOrphy[voie].active then inc(index);
     result := ansiChar(index);
//     result := char(voieOrphy[voieSynchro].numeroOrphy); ??  
end;

Function Tgraphe.isAxeY(x,y : integer) : indiceVoie;
var isaxe : boolean;
    voie : indiceVoie;
begin
    result := voieT;
    isAxe := (y>limiteCourbe.top) and
              (y<limiteCourbe.bottom) and
              (x>0) and
              (x<limiteCourbe.left);
    if not isAxe then exit;
    if analyseurLogique then begin
       for voie := voie0 to voieAnaMax do
           with voieOrphy[voie] do
               if active and
                 (voie <> voieAbscisse) and
                 (y>CourbeTop) and
                 (y<CourbeBottom) then
                    result := voie;
    end
    else result := voieYprinc;
end;

Function Tgraphe.isZeroY(x,y : integer;voie : indiceVoie) : boolean;
begin
    result := abs(y-voieOrphy[voie].B)<8
end;

function Tgraphe.changeEchelleY(ynew,yold : integer;voie : indiceVoie) : boolean;
begin with voieOrphy[voie] do begin
     result := (abs(ynew-yold)>8) and
               ((ynew-B)*(yold-B)>0);
     if result then begin // on change l'chelle ; zro reste fixe
        A := A*(ynew-B)/(yold-B); // B identique
        Maxi := (CourbeTop - B) / A;
        Mini := (CourbeBottom - B) / A;
        if positif and (mini<0) then mini := 0;
     end;
end end;

function Tgraphe.changeZeroY(ynew,yold : integer;voie : indiceVoie) : boolean;
begin with voieOrphy[voie] do begin
     result := abs(ynew-yold)>8;
     if result then begin
     // on dcale le zro et l'chelle reste fixe
        B := B+ynew-yold;  // A identique
        Maxi := (CourbeTop - B) / A;
        Mini := (CourbeBottom - B) / A;
        if positif and (mini<0) then mini := 0;
     end;
end end;

procedure Tgraphe.TraceCurseurs;
var i : integer;
begin
     canvas.Pen.Style := psSolid;
     canvas.Pen.mode := pmNotXor;
     canvas.Pen.Width := 1;
     for i := 1 to 2 do begin
         canvas.Pen.color := couleurCurseur[i];
         segment(curseurs[i].x, LimiteCourbe.top, curseurs[i].x, LimiteCourbe.bottom);
         segment(LimiteCourbe.left,curseurs[i].y, LimiteCourbe.right,curseurs[i].y);
     end;
     PenDefault;    
end;

procedure Tgraphe.VerifCurseurs;
begin
    if (curseurs[1].x<LimiteCourbe.left) or
       (curseurs[1].x>LimiteCourbe.right) then
       curseurs[1].x := LimiteCourbe.left+
              (LimiteCourbe.right-LimiteCourbe.left) div 3;
    if (curseurs[2].x<LimiteCourbe.left) or
       (curseurs[2].x>LimiteCourbe.right) then
       curseurs[2].x := LimiteCourbe.left+
              2*(LimiteCourbe.right-LimiteCourbe.left) div 3;
    if (curseurs[1].y<LimiteCourbe.top) or
       (curseurs[1].y>LimiteCourbe.bottom) then
       curseurs[1].y := LimiteCourbe.top+
           (LimiteCourbe.bottom-LimiteCourbe.top) div 3;
    if (curseurs[2].y<LimiteCourbe.top) or
       (curseurs[2].y>LimiteCourbe.bottom) then
       curseurs[2].y := LimiteCourbe.top+
          2*(LimiteCourbe.bottom-LimiteCourbe.top) div 3;
end;

Procedure Tgraphe.GetCurseurs(grid : TstringGrid);
var i,j,jref : integer;
    xr : array[1..2] of double;
    voie,voieY : indiceVoie;
    row : integer;
begin
    voieY := voie0;
    for voie := voie0 to voieAnaMax do
      if voieOrphy[voie].active and
        (voie <> voieAbscisse) then begin
           voieY := voie;
           break;
        end;
    jRef := 0;
    if NbreAff>NbreAffMin then
    for i := 1 to 2 do begin
        xr[i] := voieOrphy[voieAbscisse].XMonde(curseurs[i].x);
        for j := 1 to pred(NbreAff) do
            if (voieOrphy[voieAbscisse].vecteurAff[j]-xr[i])*
               (voieOrphy[voieAbscisse].vecteurAff[j-1]-xr[i])<=0 then begin
                 jRef := j;
                 break;
            end;
        curseurs[i].y := voieOrphy[voieY].XWindow(voieOrphy[voieY].vecteurAff[jRef]);
    end;
    row := 0;
    with voieOrphy[voieY] do begin
         for i := 1 to 2 do begin
             xr[i] := XMonde(curseurs[i].y);
             grid.cells[1,row] := formatUnite(xr[i]);
             grid.cells[0,row] := nom;
             inc(row);
         end;
         grid.cells[1,row] := formatUnite(xr[2]-xr[1]);
         grid.cells[0,row] := deltaMaj+nom;
         inc(row);
    end;
    with voieOrphy[voieAbscisse] do begin
         for i := 1 to 2 do begin
             xr[i] := XMonde(curseurs[i].x);
             grid.cells[1,row] := formatUnite(xr[i]);
             grid.cells[0,row] := nom;
             inc(row);
         end;
         grid.cells[1,row] := formatUnite(xr[2]-xr[1]);
         grid.cells[0,row] := deltaMaj+nom;
         inc(row);
         if nomUnite='s' then begin
            if xr[2]=xr[1] then begin
               grid.cells[0,row] := '';
               exit;
            end;
            try
            grid.cells[1,row] := Frequence.formatUnite(1/(xr[2]-xr[1]));
            grid.cells[0,row] := '1/'+deltaMaj+nom;
            except
                grid.cells[0,row] := '';
            end;
         end
         else begin
            grid.cells[1,row] := '';
            grid.cells[0,row] := '';
         end;
    end;
end;

Function TvoieCapteur.ValeurReal(V : word) : double;
begin
     result := ValeurVolt(V);
     if isLumicolor or isPhotocolor
        then begin
            try
            result := log10(tensionRef/result);
            except
            result := maxi; // saturation
            end
        end
        else result := ZeroVolt+unVolt*result;
end;

Function TvoieCapteur.ValeurVolt(V : word) : double;
begin
     result := -6+12*V/MaxiHard
end;

procedure Tcourbe.getCapteurVolt(capteurCourant : TCapteur);
var gamme : double;
begin with CapteurCourant do begin
            nom := nomC;
            nomUnite := nomUniteC;
            signif := signifC;
            aide := aideC;
            ZeroVolt := ZeroVoltC;
            UnVolt := UnVoltC;
            isLumicolor := isLumicolorC;
            isPhotocolor := isPhotocolorC;
            case NumeroOrphy of
                 8,9,10 : gamme := 25;
                 11 : gamme := 6;
                 12,13,14 : gamme := 0.25;
                 15 : gamme := 1;
                 else gamme := 1;
            end;
            miniCapteur := ZeroVoltC-UnVoltC*gamme;
            maxiCapteur := ZeroVoltC+UnVoltC*gamme;
            positif := miniC>=0;
            setMinMax(miniCapteur,maxiCapteur);
            mini := miniC;
            maxi := maxiC;
end end;

procedure TvoieCalculee.setExpression(Avalue : string);
var v : indiceVoie;
begin
    if Avalue<>'' then begin
        try
        TrimComplet(Avalue);
        for v := voie0 to graphe.voieAnaMax do
            with graphe.voieOrphy[v] do if active then
                 Parser.SetVariable(nom, valeurCourante);
        Parser.expression := Avalue;
        except
           active := false;
           showMessage('Erreur de syntaxe'+crCR+
                       'Utiliser le point dcimal'+crCR+
                       'Tension mesure : V');
        end;
    end
    else begin
       parser.expression := '';
       active := false;
    end;
end;

procedure Tcourbe.Load(Avoie: string; fichier: tMemIniFile);
begin with fichier do begin
      Couleur := readInteger(Avoie, 'Couleur', couleur);
      Motif := Tmotif(readInteger(Avoie, 'Motif', 0));
      NomUnite := ReadString(Avoie, 'Unite', NomUnite);
      Signif := ReadString(Avoie, 'Signif', '');
      Mini := ReadFloat(Avoie, 'Mini', -5);
      Maxi := ReadFloat(Avoie, 'Maxi', 5);
      MaxiCapteur := Maxi;
      MiniCapteur := Mini;
      active := ReadBool(Avoie, 'Active', False);
      isLumicolor := ReadBool(Avoie,'Lumicolor',false);
      isPhotocolor := ReadBool(Avoie,'Photocolor',false);
      Positif := ReadBool(Avoie, 'Positif', false);
   end;
end;

procedure Tcourbe.Store(Avoie: string; Fichier: tMemIniFile);
begin
with fichier do begin
      WriteInteger(Avoie, 'Couleur', couleur);
      WriteInteger(Avoie, 'Motif', Ord(motif));
      WriteString(Avoie, 'Nom', nom);
      WriteString(Avoie, 'Unite', nomUnite);
      WriteString(Avoie, 'Signif', signif);
      WriteFloat(Avoie, 'Mini', mini);
      WriteFloat(Avoie, 'Maxi', maxi);
      WriteBool(Avoie, 'Active', active);
      WriteBool(Avoie, 'Positif', positif);
end end;

procedure TvoieCalculee.Store(Avoie: string; Fichier: tMemIniFile);
begin
   inherited store(Avoie,Fichier);
   fichier.WriteString(Avoie, 'Expression', expression);
   fichier.WriteString(Avoie, 'Symb', nom);
end;

procedure Tgraphe.TraceRepere(repere: Trepere);
var
   x,largeur: integer;
begin with Canvas, repere do begin
      Pen.color := couleurRepere;
      Pen.Style := psDash;
      Pen.Width := 1;
      x := VoieOrphy[voieT].Xwindow(valeur);
      MoveTo(x, LimiteCourbe.top);
      LineTo(x, LimiteCourbe.bottom);
      largeur := textWidth(edit.text);
      edit.Left := x-(largeur div 2);
      edit.Top := (3*limiteCourbe.Top+limiteCourbe.Bottom) div 4;
      edit.autosize := largeur>80;
      if largeur<80 then edit.width := 80;
      PenDefault;
end end;{ TraceRepere }

procedure TlisteRepere.Clear;
var
  i: integer;
begin
  for i := 0 to pred(Count) do
    items[i].Free;
  inherited Clear;
end;

function TlisteRepere.Remove(Item: TRepere): integer;
begin
  Result := inherited Remove(Item);
  Item.Free;
end;

function TlisteRepere.Add(Item: TRepere): integer;
begin
  Result := inherited Add(Item);
end;

function TlisteRepere.Get(Index: integer): TRepere;
begin
  Result := TRepere(inherited Get(Index));
end;

procedure TlisteRepere.Put(Index: integer; Item: TRepere);
begin
  inherited Put(Index, Item);
end;

destructor Trepere.Destroy;
begin
    edit.free;
    inherited destroy;
end;

initialization
   Frequence := Tunite.Create;
   Frequence.NomUnite := 'Hz';
   Angle := Tunite.Create;
   Angle.NomUnite := 'rad';

end.

